# simple IOC / DI
from typing import Any, Dict
from oi_cli2.core.DI import DI_ACCMAN, DI_CFG, DI_DB, DI_DB_COOKIES, DI_HTTP, DI_LOGGER, DI_PROVIDER, DI_TEMPMAN
from oi_cli2.cli.constant import CIPHER_KEY, COOKIES_FILE, OT_FOLDER, OT_LOG, USER_CONFIG_FILE
from oi_cli2.utils.HttpUtil import HttpUtil
from oi_cli2.utils.Logger import getLogger
from oi_cli2.utils.account import AccountManager
from oi_cli2.utils.configFolder import ConfigFolder
from oi_cli2.utils.db import JsonFileDB
from oi_cli2.utils.enc import AESCipher
from oi_cli2.utils.template import TemplateManager
from .Singleton import Singleton


@Singleton
class Provider2:
  _objs: Dict[str, Any] = {}
  _fns: Dict[str, Any] = {}
  loop = 0  # 简单防止循环依赖

  def __init__(self) -> None:
    self.reg(DI_CFG, gen_cfg)
    self.reg(DI_LOGGER, gen_logger)
    self.reg(DI_HTTP, gen_http_util)
    self.reg(DI_DB, gen_json_db)
    self.reg(DI_DB_COOKIES, gen_json_db_cookies)
    self.reg(DI_ACCMAN, gen_account_manager)
    self.reg(DI_TEMPMAN, gen_template_manager)

  def reg(self, key: str, func) -> bool:
    assert key not in self._fns
    self._fns[key] = func
    return True

  def get(self, key: str) -> Any:
    self.loop += 1
    assert key in self._fns
    assert (self.loop < 100)
    if key not in self._objs:
      self._objs[key] = self._fns[key](self)

    self.loop -= 1
    return self._objs[key]


def gen_cfg(p: Provider2):
  return ConfigFolder(OT_FOLDER)


def gen_logger(o: Provider2):
  try:
    config_folder: ConfigFolder = o.get(DI_CFG)
    logger = getLogger(config_folder.get_file_path(OT_LOG))
  except Exception as e:
    print(str(e))
    exit(1)
  return logger


def gen_template_manager(o: Provider2):
  return TemplateManager(db=o.get(DI_DB))


def gen_account_manager(o: Provider2):
  return AccountManager(db=o.get(DI_DB), cipher=AESCipher(CIPHER_KEY), logger=o.get(DI_LOGGER))


def gen_json_db(o: Provider2):
  config_folder: ConfigFolder = o.get(DI_CFG)
  return JsonFileDB(config_folder.get_config_file_path(USER_CONFIG_FILE), logger=o.get(DI_LOGGER))


def gen_json_db_cookies(o: Provider2):
  config_folder: ConfigFolder = o.get(DI_CFG)
  return JsonFileDB(config_folder.get_config_file_path(COOKIES_FILE), logger=o.get(DI_LOGGER))


def gen_http_util(o: Provider2):
  return HttpUtil(logger=o.get(DI_LOGGER))



import os
from typing import Any

from oi_cli2.cli.constant import RED, DEFAULT
# mac os 的命令似乎不支持 --ignore-trailing-space
diff_cmd = "diff --brief -B --ignore-trailing-space"  # 用于比较
show_diff_cmd = "diff -B --ignore-trailing-space -y"  # 用于展示差异


# TODO 非系统命令diff, 改为 自己实现函数/注入式/支持交互
def diff_result_fn(std_in_file: str, std_out_file: str, user_out_file: str) -> None:
  diff = os.system(f"{diff_cmd} {std_out_file} {user_out_file}")
  if diff != 0:
    print(RED)
    print("==============================================================================")
    os.system(f"cat {std_in_file}")
    print("\n-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - -")
    os.system(f"{show_diff_cmd} {std_out_file} {user_out_file}")
    print("\n==============================================================================")
    print(DEFAULT)



import asyncio
import json
import logging
import os
import traceback
from typing import Tuple, cast
import click
from rich.console import Console
from rich.text import Text
from rich.table import Table
from rich.live import Live

from oi_cli2.cli.adaptor.ojman import OJManager
from oi_cli2.cli.constant import FETCH_RESULT_INTERVAL, STATE_FILE
from oi_cli2.core.DI import DI_ACCMAN, DI_LOGGER, DI_TEMPMAN
from oi_cli2.model.Account import Account
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.model.FolderState import FolderState
from oi_cli2.model.Result import SubmissionResult, status_string
from oi_cli2.utils.Provider2 import Provider2
from oi_cli2.utils.account import AccountManager
from oi_cli2.utils.template import TemplateManager

console = Console(color_system='256', style=None)


def generate_submission_table(res: SubmissionResult) -> Table:
  """Make a new submission table."""
  table = Table().grid()
  table.add_column(min_width=12)
  table.add_column()
  table.add_row("Result ID", f"{res.id}")
  # "[red]ERROR" if value < 50 else "[green]SUCCESS"
  table.add_row("Status", Text.from_ansi(f"{status_string(res)}"))
  table.add_row("Time", f"{res.time_note}")
  table.add_row("Memory", f"{res.mem_note}")
  if res.msg_txt:
    table.add_row("MSG", f"{res.msg_txt}")
  if res.url:
    table.add_row("Url", f"{res.url}")
  return table


def watch_result(oj: BaseOj, problem_url: str) -> SubmissionResult:
  return asyncio.run(async_watch_result(oj, problem_url))


async def async_watch_result(oj: BaseOj, problem_url: str) -> SubmissionResult:
  await oj.init()
  try:
    result = SubmissionResult()
    with Live(auto_refresh=False) as live:
      async for result in oj.async_get_result_yield(problem_url, time_gap=FETCH_RESULT_INTERVAL):
        live.update(generate_submission_table(result), refresh=True)
  except Exception as e:
    logger: logging.Logger = Provider2().get(DI_LOGGER)
    logger.exception(e)
  await oj.deinit()
  return result


def submit_parser() -> Tuple[str, str, str, Account, str, str]:
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)
  tm: TemplateManager = Provider2().get(DI_TEMPMAN)
  # get lang config
  if not os.path.isfile(STATE_FILE):
    raise Exception(f'STATE_FILE [{STATE_FILE}] NOT EXIST!')
  state_oj = FolderState()
  with open(STATE_FILE) as f:
    state_oj.__dict__ = json.load(f)

  oj = state_oj.oj
  up_lang = cast(str, state_oj.up_lang)

  template = tm.get_template_by_name(state_oj.oj, state_oj.template_alias)

  if template is None:
    raise Exception(f'Template not found by [{state_oj.oj},{state_oj.template_alias}]')
  source_file_name = os.path.basename(template.path)
  code_file = os.path.join('.', source_file_name)
  if not os.path.isfile(code_file):
    raise Exception(f"code_file [{code_file}] NOT EXIST!")

  account = am.get_default_account(oj)
  return oj, state_oj.id, up_lang, account, code_file, state_oj.problem_url


@click.command(name="submit")
def submit_command() -> None:
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  try:
    platform, sid, up_lang, account, code_path, problem_url = submit_parser()

    table = Table().grid()
    table.add_column(min_width=12)
    table.add_column()
    table.add_row("OJ", f"{platform}")
    table.add_row("Account", f"{account.account}")
    table.add_row("Problem ID", f"{sid}")
    table.add_row("up_lang", f"{up_lang}")
    console.print(table)

    try:
      oj: BaseOj = OJManager.createOj(platform=platform, account=account, provider=Provider2())
    except Exception as e:
      logger.exception(e)
      raise e

    if not oj.submit_code(problem_url=problem_url, language_id=up_lang, code_path=code_path):
      raise Exception(f'submit failed, account={account.account}')
    console.print("[green]Submitted")
    watch_result(oj, problem_url)
  except KeyboardInterrupt:
    logger.info("Interrupt by user")
  except Exception:
    logger.error(traceback.format_exc())


@click.command(name="result")
def result_command() -> None:
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  platform, sid, up_lang, account, code_path, problem_url = submit_parser()
  table = Table().grid()
  table.add_column(min_width=12)
  table.add_column()
  table.add_row("OJ", f"{platform}")
  table.add_row("Account", f"{account.account}")
  table.add_row("Problem ID", f"{sid}")
  table.add_row("up_lang", f"{up_lang}")
  table.add_row("Problem Url", f"{problem_url}")
  console.print(table)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform, account=account, provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  logger.debug(problem_url)

  watch_result(oj, problem_url)



from typing import List

# Analyze 是紧密依赖 不采用依赖注入？
from oi_cli2.model.Analyze import Analyze
# 依赖注入
from oi_cli2.utils.db import JsonFileDB
# 静态配置
from oi_cli2.utils.consts.ids import Ids


class AnalyzeManager:

  def __init__(self, db: JsonFileDB):
    self.db = db
    self.keys = ['platform', 'alias', 'path', 'compilation', 'execute', 'clean', 'default']

  def _get_analyze_list(self) -> List[Analyze]:
    analyze_list: List[dict] = self.db.load(Ids.analyze) or []
    return list(map(lambda d: Analyze().dict_init(d), analyze_list))

  def _set_analyze_list(self, analyze_list: List[Analyze]):
    analyze_list.sort(key=lambda temp0: (temp0.platform, -temp0.default, temp0.template_alias))
    self.db.save(Ids.analyze, list(map(lambda d: d.__dict__, analyze_list)))

  def get_list(self) -> List[Analyze]:
    return self._get_analyze_list()

  def set_default(self, index: int):
    analyze_list: List[Analyze] = self._get_analyze_list()
    assert 0 <= index < len(analyze_list)
    for i in range(len(analyze_list)):
      if i == index:
        analyze_list[i].default = True
      elif analyze_list[i].platform == analyze_list[index].platform:
        analyze_list[i].default = False

    self._set_analyze_list(analyze_list)

  def delete_analyze(self, index):
    analyze_list: List[Analyze] = self._get_analyze_list()
    assert 0 <= index < len(analyze_list)
    if analyze_list[index].default:
      for i in range(len(analyze_list)):
        if i == index:
          continue
        if analyze_list[i].platform == analyze_list[index].platform:
          analyze_list[i].default = True
          break

    del analyze_list[index]
    self._set_analyze_list(analyze_list)

  # set default if no platform there
  def add_analyze(self, platform, submit_lang, template_alias, class_path):
    analyze_list: List[Analyze] = self._get_analyze_list()

    is_default = True
    for item in analyze_list:
      if item.platform == platform and item.default:
        is_default = False
        break

    analyze_list.append(Analyze().initial(platform, submit_lang, template_alias, class_path, default=is_default))
    self._set_analyze_list(analyze_list)

  def modify_submit_lang(self, index: int, value: str):
    analyze_list: List[Analyze] = self._get_analyze_list()
    assert 0 <= index < len(analyze_list)
    analyze_list[index].submit_lang = value
    self._set_analyze_list(analyze_list)

  def modify_template_alias(self, index: int, value: str):
    analyze_list: List[Analyze] = self._get_analyze_list()
    assert 0 <= index < len(analyze_list)
    analyze_list[index].template_alias = value
    self._set_analyze_list(analyze_list)

  def modify_class_path(self, index: int, value: str):  # 实例class 文件
    analyze_list: List[Analyze] = self._get_analyze_list()
    assert 0 <= index < len(analyze_list)
    analyze_list[index].class_path = value
    self._set_analyze_list(analyze_list)



import logging
from typing import List, Optional

from oi_cli2.model.Account import Account
# 依赖注入
from oi_cli2.utils.db import JsonFileDB
from oi_cli2.utils.enc import AESCipher
# 静态配置
from oi_cli2.utils.consts.ids import Ids


class AccountManager:

  def __init__(self, db: JsonFileDB, cipher: AESCipher, logger=logging):
    self.db = db
    self.cipher = cipher
    self.logger = logger

  def _get_account_list(self) -> List[Account]:
    acc_list: List[dict] = self.db.load(Ids.account) or []
    return list(map(lambda d: Account().dict_init(d), acc_list))

  def _set_account_list(self, acc_list: List[Account]):
    acc_list.sort(key=lambda acc0: (acc0.platform, -acc0.default, acc0.account, acc0.password))
    self.db.save(Ids.account, list(map(lambda d: d.__dict__, acc_list)))

  def get_list(self) -> List[Account]:
    return self._get_account_list()

  def set_default(self, index: int):
    accs: List[Account] = self._get_account_list()
    assert 0 <= index < len(accs)
    for i in range(len(accs)):
      if i == index:
        accs[i].default = True
      elif accs[i].platform == accs[index].platform:
        accs[i].default = False

    self._set_account_list(accs)

  def get_default_account(self, platform: str) -> Account:
    accs: List[Account] = self._get_account_list()
    for i in range(len(accs)):
      if accs[i].platform == platform and accs[i].default:
        return accs[i]
    raise Exception(f'Account Not Found int Platform [{platform}]')

  def get_account(self, platform: str, account: str) -> Optional[Account]:
    accs: List[Account] = self._get_account_list()
    for i in range(len(accs)):
      if accs[i].platform == platform and accs[i].account == account:
        return accs[i]

    return None

  def modify(self, platform: str, account: str, password=None, default=None) -> bool:
    modified = False
    accs: List[Account] = self._get_account_list()
    for item in accs:
      if item.platform == platform:
        if item.account == account:
          if password is not None:
            item.password = self.cipher.encrypt(password)
          if default:
            item.default = True
          modified = True
        elif default:
          item.default = False
    self._set_account_list(accs)
    return modified

  # Delete
  def delete(self, platform: str, account: str) -> bool:
    accs: List[Account] = self._get_account_list()
    for i in range(len(accs)):
      acc = accs[i]
      if acc.account != account or acc.platform != platform:
        continue
      if acc.default:
        # set new default
        for j in range(len(accs)):
          if j == i:
            continue
          if accs[i].platform == accs[j].platform:
            accs[i].default = True
            break
      del accs[i]
      self._set_account_list(accs)
      return True

    return False

  def new(self, platform, account, password, default=False) -> bool:
    accs: List[Account] = self._get_account_list()
    self.logger.debug("platform = %s, account = %s, default = %s", platform, account, default)
    has_default = False
    for item in accs:
      if item.platform == platform and item.account == account:
        return False
    for item in accs:
      if item.platform == platform and item.default:
        has_default = True
        break
    # first account in platform
    if not has_default:
      default = True
    accs.append(Account().initial(platform=platform,
                                  account=account,
                                  password=self.cipher.encrypt(password),
                                  default=default))
    if default:
      for item in accs:
        if item.platform == platform and item.account != account:
          item.default = False

    self._set_account_list(accs)
    return True



#!/usr/bin/env python3
import logging
import os
import sys
import json
from typing import List, Type
from oi_cli2.cli.constant import CIPHER_KEY, OT_FOLDER, USER_CONFIG_FILE

from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.model.ParseProblemResult import ParsedProblemResult
from oi_cli2.model.ProblemMeta import ProblemMeta
from oi_cli2.model.TestCase import TestCase
from oi_cli2.model.FolderState import FolderState

from oi_cli2.utils.FileUtil import FileUtil
from oi_cli2.utils.configFolder import ConfigFolder

# file_util can be any thing , everything is file
from oi_cli2.utils.template import TemplateManager
from oi_cli2.utils.force_symlink import force_symlink

# def createDir(oj: BaseOj, problem_id: str, problem: ProblemMeta, file_util: Type[FileUtil], logger,
#               template_manager: TemplateManager, config_folder: ConfigFolder):
#   template = template_manager.get_platform_default(type(oj).__name__)
#   if template is None:
#     print(type(oj).__name__ + ' has no default template, run ./oiTerminal.py config first')
#     logger.warn(f'{type(oj).__name__} parse problem when no template set')
#     return None
#
#   result = oj.problem(problem_id)
#   test_cases: List[TestCase] = result.test_cases
#   directory = config_folder.get_file_path(os.path.join('dist', type(oj).__name__, result.file_path))
#
#   for i in range(len(test_cases)):
#     file_util.write(config_folder.get_file_path(os.path.join(directory, f'in.{i}')), test_cases[i].in_data)
#     file_util.write(config_folder.get_file_path(os.path.join(directory, f'out.{i}')), test_cases[i].out_data)
#
#   # if code file exist not cover code
#   if not os.path.exists(config_folder.get_file_path(os.path.join(directory, os.path.basename(template.path)))):
#     file_util.copy(config_folder.get_file_path(template.path),
#                    config_folder.get_file_path(os.path.join(directory, os.path.basename(template.path))))
#   # TODO 生成state.json ( 提供 自定义字段)
#   TEST_PY = 'test.py'
#   SUBMIT_PY = 'submit.py'
#   STATE_FILE = 'state.json'
#   # symlink test.py submit.py
#   RELATIVE_CLI_FOLDER = '../../../../'
#   force_symlink(os.path.join(RELATIVE_CLI_FOLDER, TEST_PY),
#                 config_folder.get_file_path(os.path.join(directory, TEST_PY)))
#   force_symlink(os.path.join(RELATIVE_CLI_FOLDER, SUBMIT_PY),
#                 config_folder.get_file_path(os.path.join(directory, SUBMIT_PY)))
#
#   # TODO provide more info, like single test and
#   # generate state.json
#   folder_state = FolderState(oj=type(oj).__name__,
#                              sid=problem_id,
#                              template_alias=template.alias,
#                              up_lang=template.uplang)  # TODO get data from analyzer
#   with open(config_folder.get_file_path(os.path.join(directory, STATE_FILE)), "w") as statejson:
#     json.dump(folder_state.__dict__, statejson)
#     statejson.close()
#
#   return directory

# def main(argv: List[str], logger: logging, folder=OT_FOLDER):
#   config_folder = ConfigFolder(folder)
#   user_config_path = config_folder.get_config_file_path(USER_CONFIG_FILE)
#
#   http_util = HttpUtil(logger=logger)
#   dbIns = JsonFileDB(file_path=user_config_path, logger=logger)
#   template_manager = TemplateManager(db=dbIns)
#   account_manager = AccountManager(db=dbIns, cipher=AESCipher(CIPHER_KEY))
#
#   if argv[0] == Platforms.codeforces:
#     try:
#       from oi_cli2.custom.Codeforces.Codeforces import Codeforces
#       oj: BaseOj = Codeforces(http_util=http_util,
#                               logger=logger,
#                               account=account_manager.get_default_account(Codeforces.__name__),
#                               html_tag=HtmlTag(http_util))
#     except Exception as e:
#       logger.exception(e)
#       raise e
#   else:
#     raise Exception('Unknown Platform')
#
#   directory = createDir(
#       oj=oj,
#       problem_id=argv[1],
#       ProblemMeta=None,  # TODO support
#       file_util=FileUtil,
#       logger=logger,
#       template_manager=template_manager,
#       config_folder=config_folder)
#
#   if directory is None:
#     return None
#   # TODO switch directory
#   print(directory)
#
#   start_terminal(config_folder.get_file_path(os.path.join(directory)))

# if __name__ == '__main__':
#   main(sys.argv, folder=OT_FOLDER)



from typing import List
from enum import Enum

from oi_cli2.model.TestCase import TestCase


class Problem:

  class Status(Enum):
    AC = 'AC'
    FAILED = 'FAILED'
    NOTVIS = 'NOTVIS'

  title: str
  status: Status
  test_cases: List[TestCase]

  def __init__(self,
               pid: str,
               oj: str,
               description: str = '',
               test_cases: List[TestCase] = None,
               time_limit: str = '',
               mem_limit: str = '',
               url: str = '',
               html: str = ''):
    self.id: str = pid
    self.oj: str = oj
    self.description: str = description
    if test_cases is None:
      # don't place it on arguments
      self.test_cases = []
    else:
      self.test_cases = test_cases
    self.time_limit: str = time_limit
    self.mem_limit: str = mem_limit
    self.url: str = url
    self.html: str = html

  def __repr__(self):
    return Problem.__name__ + str(self.__dict__)

  def __str__(self):
    return self.__repr__()



# TODO not important support lowercase and more, cf,codeforces,Codeforces -> Codeforces
import logging
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.utils.Singleton import Singleton


@Singleton
class OJUtil(object):

  def __init__(self):
    self.short2class = {}

    from oi_cli2.cli.adaptor.Codeforces import Codeforces
    self.reg(Codeforces)
    # TODO atcoder

  def reg(self, oj_class: BaseOj):
    assert False
    # for short_name in oj_class.short_name:
    #   if short_name in self.short2class:
    #     logging.error(f'{short_name} already registered')
    #   else:
    #     self.short2class[short_name] = oj_class



#!/usr/bin/env python3
import click
from oi_cli2.core.DI import DI_DB, DI_LOGGER
from oi_cli2.cli.constant import OT_FOLDER, USER_CONFIG_FILE
from oi_cli2.cli.template import template
from oi_cli2.cli.account import account
from oi_cli2.utils.configFolder import ConfigFolder
from oi_cli2.utils.db import JsonFileDB
from oi_cli2.utils.Provider2 import Provider2
import logging


@click.group()
@click.pass_context
def config(ctx) -> None:
  """Config environment """
  # TODO make it provider?
  config_folder = ConfigFolder(OT_FOLDER)
  try:
    logger: logging.Logger = Provider2().get(DI_LOGGER)
  except Exception as e:
    print(str(e))
    exit(1)
  ctx.obj[DI_LOGGER] = logger
  ctx.obj[DI_DB] = JsonFileDB(config_folder.get_config_file_path(USER_CONFIG_FILE), logger=logger)


config.add_command(account)
config.add_command(template)



from datetime import datetime, timedelta
# from tests.mock.MockHttpUtil import MockHttpUtil
from rich.console import Console
from rich.table import Table
from rich.style import Style

from codeforces_core.contest_list import ContestList

timeFmt = "%b/%d/%Y %H:%M"


def printData(cl: ContestList):
  # print(html)
  cur = cl.upcomming
  his = cl.history
  console = Console()

  table = Table(title="Current or upcoming contests")
  table.add_column("Name", style="cyan", no_wrap=False)
  # table.add_column("Writers", style="magenta")
  table.add_column("Start")
  table.add_column("Length")
  table.add_column("Before Start")
  table.add_column("Registered")
  table.add_column("Id", style="magenta")

  for item in cur:
    startTime = datetime.fromtimestamp(item.start)
    timediff = startTime - datetime.now()
    if not item.participants:  # 未开放注册
      bgcolor = None
    elif item.registered:  # 已注册
      bgcolor = 'dark_green'
    else:  # 开放注册,未注册
      bgcolor = 'grey30'
    table.add_row(
        item.title,
        # item["writers"],  # tds[1].get_text().strip(),
        # https://strftime.org/
        # Jan/27/2022 17:35 tds[2].get_text().strip(),
        startTime.strftime(timeFmt),
        item.length,  # tds[3].get_text().strip(),
        str(timedelta(days=timediff.days, seconds=timediff.seconds)) if timediff.days >= 0 else 'Started',
        item.participants,  # tds[5].get_text().strip(),
        str(item.id),  # tds[5].get_text().strip(),
        style=Style(bgcolor=bgcolor))

  console.print(table)

  # history
  table = Table(title="History contests (Recent 10)")
  table.add_column("Name", style="cyan", no_wrap=False)
  # table.add_column("Writers", style="magenta")
  table.add_column("Start")
  table.add_column("Length")
  table.add_column("Registered")
  table.add_column("Id", style="magenta")

  for i in range(min(10, len(his))):
    item = his[i]
    table.add_row(
        item.title,  # tds[0].get_text().strip(),
        # item["writers"],  # tds[1].get_text().strip(),
        # https://strftime.org/
        # Jan/27/2022 17:35 tds[2].get_text().strip(),
        datetime.fromtimestamp(item.start).strftime(timeFmt),
        item.length,  # tds[3].get_text().strip(),
        item.participants,  # tds[5].get_text().strip(),
        str(item.id),  # tds[5].get_text().strip(),
    )

  console.print(table)



from typing import Dict

from oi_cli2.model.Problem import Problem


class Contest:

  def __init__(self, oj: str, cid: str):
    self._oj: str = oj
    self._id = cid
    self._url: str = ''
    self._name: str = ''
    self.problems: Dict[str, Problem] = {}

  @property
  def id(self):
    return self._id

  @property
  def name(self):
    return self._name

  @name.setter
  def name(self, value):
    self._name = value

  @property
  def url(self):
    return self._url

  @url.setter
  def url(self, value):
    self._url = value

  @property
  def oj(self):
    return self._oj

  @oj.setter
  def oj(self, value):
    self._oj = value



from datetime import datetime, timedelta
# from tests.mock.MockHttpUtil import MockHttpUtil
from rich.console import Console
from rich.table import Table
from rich.style import Style

from ac_core.contest import ContestListPage

timeFmt = "%b/%d/%Y %H:%M"


def min2str(min: int) -> str:

  # prefix0
  _ = lambda v: f'0{v}'[-2:]

  if min <= 60:
    return _(min)
  if min <= 24 * 60:
    return f'{_(min//60)}:{_(min%60)}'
  return f'{min//60//24} day {_((min//60)%24)}:{_(min%60)}'


def full_url(url: str) -> str:
  return ('https://atcoder.jp' + url) if url.startswith('/') else url


def printData(cl: ContestListPage):
  # print(html)
  cur = cl.up_comming
  his = cl.recent
  console = Console()

  table = Table(title="Upcoming")
  table.add_column("Name", style="cyan", no_wrap=False)
  # table.add_column("Writers", style="magenta")
  table.add_column("Start")
  table.add_column("Before Start")
  table.add_column("Length")
  table.add_column("url", style="magenta")

  for item in cur:
    startTime = datetime.fromtimestamp(item.start_timestamp)
    timediff = startTime - datetime.now()
    table.add_row(
        item.name,
        # item["writers"],  # tds[1].get_text().strip(),
        # https://strftime.org/
        # Jan/27/2022 17:35 tds[2].get_text().strip(),
        startTime.strftime(timeFmt),
        str(timedelta(days=timediff.days, seconds=timediff.seconds)) if timediff.days >= 0 else 'Started',
        min2str(item.duration),  # tds[3].get_text().strip(),
        full_url(item.url),  # tds[5].get_text().strip(),
    )

  console.print(table)

  # history
  table = Table(title="History contests (Recent 10)")
  table.add_column("Name", style="cyan", no_wrap=False)
  # table.add_column("Writers", style="magenta")
  table.add_column("Start")
  table.add_column("Length")
  table.add_column("url", style="magenta")

  for i in range(min(10, len(his))):
    item = his[i]
    table.add_row(
        item.name,  # tds[0].get_text().strip(),
        # item["writers"],  # tds[1].get_text().strip(),
        # https://strftime.org/
        # Jan/27/2022 17:35 tds[2].get_text().strip(),
        datetime.fromtimestamp(item.start_timestamp).strftime(timeFmt),
        min2str(item.duration),  # tds[3].get_text().strip(),
        full_url(item.url),  # tds[5].get_text().strip(),
    )

  console.print(table)



import asyncio
from typing import Any, AsyncGenerator

# def iter_over_async(
#     async_generator: AsyncGenerator,
#     loop: asyncio.AbstractEventLoop = None,
# ):
#   ait = async_generator.__aiter__()
#   if loop == None:
#     loop = asyncio.get_event_loop()
#
#   async def get_next():
#     try:
#       obj = await ait.__anext__()
#       print('obj = ', obj)
#       return False, obj
#     except StopAsyncIteration:
#       return True, None
#
#   while True:
#     done, obj = loop.run_until_complete(get_next())
#     if done:
#       break
#     print('iter yield obj = ', obj)
#     yield obj
#
#   # ait = async_generator.__aiter__()
#
#   # async def get_next() -> tuple[bool, Any]:
#   #   try:
#   #     obj = await ait.__anext__()
#   #     done = False
#
#   #   except StopAsyncIteration:
#   #     obj = None
#   #     done = True
#
#   #   return done, obj
#
#   # while True:
#   #   done, obj = asyncio.run_coroutine_threadsafe(get_next(), loop).result()
#
#   #   if done:
#   #     break
#   #  yield obj
#



import os

# recursive search folder_name in dirpath and it's parent path


class ConfigFolder(object):

  def __init__(self, folder_name: str):
    self.folder_name = folder_name

  def get_root_folder(self):
    abs_path = os.path.abspath("./")
    while True:
      test_dir_path = os.path.join(abs_path, self.folder_name)
      if os.path.isdir(test_dir_path):
        return abs_path
      parent_path = os.path.abspath(os.path.join(abs_path, os.pardir))
      # Root folder's parent = Root folder
      if parent_path == abs_path:
        return None
      abs_path = parent_path

  def get_config_file_path(self, file_path: str):
    abs_folder = self.get_root_folder()
    if abs_folder is None:
      raise Exception('Not a oi folder, please run `oi init` first')
    return os.path.join(abs_folder, self.folder_name, file_path)

  def get_file_path(self, file_path: str):
    abs_folder = self.get_root_folder()
    if abs_folder is None:
      raise Exception('Not a oi folder, please run `oi init` first')

    return os.path.join(abs_folder, file_path)



class Account:
  platform: str
  account: str
  password: str
  default: bool

  def __init__(self):
    self.platform = ''
    self.account = ''
    self.password = ''
    self.default = False

  def initial(self, platform, account, password, default=False):
    self.platform = platform
    self.account = account
    self.password = password
    self.default = default
    return self

  def dict_init(self, d):
    self.__dict__ = d
    if not hasattr(self, 'platform'):
      self.platform = ''
    if not hasattr(self, 'account'):
      self.account = ''
    if not hasattr(self, 'password'):
      self.password = ''
    if not hasattr(self, 'default'):
      self.default = False
    return self

  def __repr__(self):
    return "<Account platform:%s username:%s>" % (self.platform, self.account)

  def __str__(self):
    return f"Account({self.platform},{self.account},{self.default})"



from oi_cli2.core.DI import DI_CFG, DI_LOGGER
from oi_cli2.model.Account import Account
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.utils.Provider2 import Provider2
from oi_cli2.utils.configFolder import ConfigFolder


def CodeforcesGen(account: Account, provider: Provider2) -> BaseOj:
  from oi_cli2.cli.adaptor.Codeforces.Codeforces import Codeforces
  from codeforces_core.httphelper import HttpHelper
  config_folder: ConfigFolder = provider.get(DI_CFG)
  cookie_jar_path = config_folder.get_config_file_path(account.platform + '_' + account.account + '_cookies')
  # token_path = config_folder.get_config_file_path(account.platform + '_' + account.account + '_tokens')
  http = HttpHelper(token_path='', cookie_jar_path=cookie_jar_path)
  oj: BaseOj = Codeforces(http_util=http, logger=provider.get(DI_LOGGER), account=account)

  return oj



import requests


class HttpUtil(object):

  def __init__(self, headers=None, logger=None):
    self._headers = headers or {
        'User-Agent':
        'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36'
    }
    self._request = requests.Session()
    self._timeout = (10, 20)  # connect timeout , read timeout
    self._logger = logger
    if self._headers:
      self._request.headers.update(self._headers)

  def get(self, url, allow_redirects: bool = True):
    # TODO ? fix kwargs timeout 优先级大于内置_timeout ?
    return self._request.get(url, timeout=self._timeout, allow_redirects=allow_redirects)

  def post(self, url, data=None, allow_redirects: bool = True):
    try:
      return self._request.post(url, data, timeout=self._timeout, allow_redirects=allow_redirects)
    except Exception as e:
      self._logger.exception(e)
      return None

  @property
  def headers(self):
    return self._request.headers

  @property
  def cookies(self):
    return self._request.cookies

  @staticmethod
  def abs_url(remote_path, oj_prefix):
    """

    :param remote_path: 原本的文件路径，可能是相对路径也可能是http或https开始的路径
    :param oj_prefix: oj的static文件前缀
    :return: 文件名，原本的补全之后的路径
    """
    if not remote_path.startswith('http://') and not remote_path.startswith('https://'):
      remote_path = oj_prefix.rstrip('/') + '/' + remote_path.lstrip('/')
    file_name = str(str(remote_path).split('/')[-1])
    return file_name, remote_path



from dataclasses import dataclass, field
from typing import List
from enum import Enum

from oi_cli2.model.TestCase import TestCase


@dataclass
class ParsedProblemResult(object):

  class Status(Enum):
    AC = 'AC'
    FAILED = 'FAILED'
    NOTVIS = 'NOTVIS'

  status: Status = Status.NOTVIS
  title: str = ''
  test_cases: List[TestCase] = field(default_factory=lambda: [])
  id: str = ''
  oj: str = ''
  description: str = ''
  time_limit: str = ''
  mem_limit: str = ''
  url: str = ''
  html: str = ''
  file_path: str = ''



from dataclasses import dataclass
from enum import Enum
from oi_cli2.cli.constant import GREEN, RED, YELLOW, DEFAULT


@dataclass
class SubmissionResult:

  class Status(Enum):
    PENDING, RUNNING, AC, RE, CE, WA, TLE, MLE, IDLE, UNKNOWN = range(10)

  id: str = '0'
  cur_status: Status = Status.PENDING
  url: str = ''  # print for user
  quick_key: str = ''  # for refetch result
  state_note: str = '0'
  time_note: str = ''
  mem_note: str = ''
  msg_txt: str = ''


def status_string(result: SubmissionResult) -> str:
  return {
      SubmissionResult.Status.PENDING: YELLOW + 'Pending...' + DEFAULT,
      SubmissionResult.Status.RUNNING: GREEN + 'Running...' + DEFAULT,
      SubmissionResult.Status.AC: GREEN + 'AC' + DEFAULT,
      SubmissionResult.Status.CE: RED + 'CE' + DEFAULT,
      SubmissionResult.Status.RE: RED + 'RE' + DEFAULT,
      SubmissionResult.Status.WA: RED + 'WA' + DEFAULT,
      SubmissionResult.Status.TLE: RED + 'TLE' + DEFAULT,
      SubmissionResult.Status.MLE: RED + 'MLE' + DEFAULT,
      SubmissionResult.Status.IDLE: RED + 'IDLENESS_LIMIT_EXCEEDED' + DEFAULT,
      SubmissionResult.Status.UNKNOWN: RED + 'UNKNOWN RESULT' + DEFAULT,
  }[result.cur_status]



import logging
import os
import traceback

from oi_cli2.utils.Singleton import Singleton

from oi_cli2.cli.constant import APP_NAME


@Singleton
class LogConfig:

  def __init__(self):
    self.env = os.getenv('OITERMINAL_ENV')
    if self.env is None:
      self.env = 'production'  # default

  def is_production(self):
    return self.env == 'production'

  def is_dev(self):
    return self.env == 'dev'


def getLogger(logger_path):
  try:
    os.makedirs(os.path.dirname(logger_path))
  except FileExistsError:
    pass
  except Exception as e:
    print(e)
    traceback.print_exc()

  # 调用库的 logging 名是 {APP_NAME}.xxx
  logger = logging.getLogger(APP_NAME)
  # basic level shoud not larger than handler
  logger.setLevel(logging.DEBUG)
  # remove default handler
  logging.getLogger().handlers.clear()

  # file
  fh = logging.FileHandler(logger_path)
  fileformatter = logging.Formatter(
      '[%(asctime)s %(name)s %(levelname)s %(filename)s %(funcName)s %(lineno)d]: %(message)s')
  fh.setFormatter(fileformatter)
  fh.setLevel(logging.DEBUG)
  logger.addHandler(fh)

  # stream warning
  sh = logging.StreamHandler()
  streamformatter = logging.Formatter('[%(levelname)s]: %(message)s')
  sh.setFormatter(streamformatter)
  sh.setLevel(logging.INFO if LogConfig().is_production() else logging.DEBUG)
  logger.addHandler(sh)

  return logger



#!/usr/bin/env python3
import click
from .reg_list import reg_list
from oi_cli2.cli.adaptor.ojman import OJManager

from oi_cli2.cli.config import config
from oi_cli2.cli.contest import contest
from oi_cli2.cli.init import init
from oi_cli2.cli.submit import submit_command, result_command
from oi_cli2.cli.lang import lang_command
from oi_cli2.cli.t3st import t3st_command
from oi_cli2.cli.completion import completion_command


@click.group()
@click.pass_context
def main(ctx={}):
  ctx.obj = {}
  #     description='oi-cli for AtCoder & Codeforces')
  # TODO problem contest test submit
  # TODO move to auto test

  # exit(1)
  # if ops[0] == 'problem':
  #   from oiTerminal.cli import problem
  #   problem.main(argv=args.args, logger=logger, folder=folder)
  # else:
  #   parser.print_help()


for [key, fn] in reg_list.items():
  OJManager.regOj(key, fn)

main.add_command(init)
main.add_command(config)
main.add_command(contest)
main.add_command(t3st_command)
main.add_command(submit_command)
main.add_command(result_command)
main.add_command(lang_command)
main.add_command(completion_command)



# TODO dataclasses, for better debug


class FolderState:
  oj: str
  id: str  # string id
  cid: str  # contest id
  pid: str  # problem id
  problem_url: str
  up_lang: str
  template_alias: str

  def __init__(self,
               oj: str = '',
               sid: str = '',
               cid: str = '',
               pid: str = '',
               problem_url='',
               up_lang: str = '',
               template_alias: str = ''):
    self.oj = oj
    self.id = sid
    self.cid = cid
    self.pid = pid
    self.problem_url = problem_url
    self.up_lang = up_lang
    self.template_alias = template_alias



from hashlib import md5
from base64 import b64decode, b64encode

from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes
from Crypto.Util.Padding import pad, unpad


class AESCipher:

  def __init__(self, key: str):
    self.key = md5(key.encode('utf8')).digest()

  def encrypt(self, data: str) -> str:
    iv = get_random_bytes(AES.block_size)
    cipher = AES.new(self.key, AES.MODE_CBC, iv)
    return b64encode(iv + cipher.encrypt(pad(data.encode('utf-8'), AES.block_size))).decode('utf-8')

  def decrypt(self, data: str) -> str:
    raw = b64decode(data)
    cipher = AES.new(self.key, AES.MODE_CBC, raw[:AES.block_size])
    return unpad(cipher.decrypt(raw[AES.block_size:]), AES.block_size).decode('utf-8')



from typing import Any, Union, AsyncIterator

from oi_cli2.model.LangKV import LangKV
from oi_cli2.model.ParseProblemResult import ParsedProblemResult
from oi_cli2.model.ProblemMeta import ContestMeta, ProblemMeta
from oi_cli2.model.Result import SubmissionResult


class BaseOj(object):

  def __init__(self):
    pass

  async def init(self) -> None:
    pass

  async def deinit(self) -> None:
    pass

  def problem_by_id(self, problem_id: str) -> ParsedProblemResult:
    assert (False)

  async def async_problem(self, problem: ProblemMeta) -> ParsedProblemResult:
    assert (False)

  def problem(self, problem: ProblemMeta) -> ParsedProblemResult:
    assert (False)

  def login_website(self, force: bool = False) -> bool:
    assert (False)

  def reg_contest(self, cid: str) -> bool:
    assert (False)

  def submit_code(self, problem_url: str, language_id: str, code_path: str) -> Union[bool, Any]:
    assert (False)

  async def async_get_result_yield(self, problem_url: str, time_gap: float = 1) -> AsyncIterator[SubmissionResult]:
    raise NotImplementedError
    yield 0

  def get_language(self) -> LangKV:
    assert (False)

  def print_contest_list(self) -> bool:
    assert (False)

  def print_friends_standing(self, cid: str) -> None:
    assert (False)

  async def async_get_contest_meta(self, cid: str) -> ContestMeta:
    assert (False)

  def get_contest_meta(self, cid: str) -> ContestMeta:
    assert (False)

  @staticmethod
  def account_required() -> bool:
    assert (False)

  @staticmethod
  def support_contest():
    assert (False)

  def cid2url(self, cid: str) -> str:
    assert (False)



#!/usr/bin/env python3
import datetime
import json
import logging
import os
import shutil
import subprocess
import traceback
import shlex
import psutil
import time
from typing import List, Callable
import click
from rich.console import Console
from rich.table import Table
from oi_cli2.cli.constant import (
    APP_NAME,
    IN_PREFIX,
    OUT_PREFIX,
    STATE_FILE,
    TEST_FOLDER,
)
from oi_cli2.utils.Provider2 import Provider2
from oi_cli2.core.DI import DI_LOGGER, DI_TEMPMAN
from oi_cli2.model.FolderState import FolderState
from oi_cli2.model.Template import Template
from oi_cli2.utils.diffTool import diff_result_fn
from oi_cli2.utils.template import TemplateManager

console = Console(color_system="256", style=None)

applogger: logging.Logger = Provider2().get(DI_LOGGER)
logger = logging.getLogger(APP_NAME + ".tester")


def tester(
    in_out_list: list[tuple[str, str]],
    work_folder: str,
    test_files: List[str],
    template: Template,
    diff_fn: Callable[[str, str, str], None],
):
    """
    all path is abspath

    - copy code into work_folder
    - run compilation command
    - run binary with std_in and user output into work_folder
    - compare std_out with user_out
    """
    print()
    # makefolder & mv code 2 folder
    for code_path in test_files:
        fname = os.path.split(code_path)[-1]
        if not os.path.exists(code_path):
            raise FileNotFoundError(f"'{code_path}' not found!")
        shutil.copy(code_path, os.path.join(work_folder, fname))
    # compile
    os.chdir(work_folder)
    logger.info(f"Tester folder: {work_folder}")
    logger.info("Start compiling.")
    logger.info(f"Run [{template.compilation}]")
    if os.system(template.compilation) != 0:
        logger.error("Complete Failed.")
        return
    logger.info("Successful compilation.")
    # run  "" not better than 'time' in bash but worse is better :-)
    # TODO 指定测试后缀
    for std_in_file, std_out_file in in_out_list:
        user_out_file = os.path.join(work_folder, os.path.split(std_out_file)[-1])
        start_time = datetime.datetime.now()
        # command = f'"{template.execute}" < "{std_in_file}" > "{user_out_file}"'
        # logger.debug(command)
        # os.system(command)
        try:
            args = shlex.split(template.execute)
            logger.info(f"Execute: [{args}]")
            logger.debug(f"in: [{std_in_file}]")
            logger.debug(f"out: [{user_out_file}]")
            # 使用subprocess运行二进制程序
            process = subprocess.Popen(
                args,
                stdin=open(std_in_file),
                stdout=open(user_out_file, "w"),
                stderr=subprocess.PIPE,
                text=True,
            )
            # 获取程序执行的进程ID
            pid = process.pid
            virtual_memory_size_mb = 0
            # 这个比 ru_maxrss 更和cf的一致
            while process.poll() is None:  # 进程存在时
                # 使用psutil获取进程的虚拟内存信息
                p = psutil.Process(pid)  # noqa: F821
                virtual_memory_info = p.memory_info()
                # 获取虚拟内存大小（以字节为单位）
                virtual_memory_size = virtual_memory_info.vms
                virtual_memory_size_mb = round(virtual_memory_size / 1024 / 1024, 2)
                # print( f"Virtual Memory Size of the binary program: {virtual_memory_size_mb} MB ")
                time.sleep(0.1)  # 暂停一秒钟，可根据需要调整

            # 等待程序执行完成
            stdout, stderr = process.communicate()  # stdout is None (to the file)
            if stderr != "":
                print("stderr:", stderr)
            return_code = process.returncode
            if return_code != 0:
                print("return code:", return_code)
        except Exception as e:
            print(f"Error running binary program: {e}")
            return None

        end_time = datetime.datetime.now()

        table = Table().grid()
        table.add_column(min_width=20)
        table.add_column()
        table.add_row(
            "TestCase",
            f"{os.path.split(std_in_file)[-1]} => {os.path.split(std_out_file)[-1]}",
        )
        table.add_row(
            "Time spend",
            f"[green]{(end_time - start_time).total_seconds()} s[/green]",
        )
        # TODO COMPARE TIME
        if virtual_memory_size_mb != 0:
            table.add_row(
                "Virtual Memory", f"[green]{virtual_memory_size_mb} MB[/green]"
            )
        console.print(table)
        diff_fn(std_in_file, std_out_file, user_out_file)

    os.chdir("../")


# t3st is test, don't trigger pytest
def t3st_main(in_out_list: list[tuple[str, str]]) -> bool:
    tm: TemplateManager = Provider2().get(DI_TEMPMAN)

    # get language config
    if not os.path.isfile(STATE_FILE):
        raise Exception(f"STATE_FILE [{STATE_FILE}] NOT EXIST!")
    state_oj = FolderState()
    with open(STATE_FILE) as f:
        state_oj.__dict__ = json.load(f)
    logger.debug(f"Loaded state:{state_oj}")

    template = tm.get_template_by_name(
        platform=state_oj.oj, name=state_oj.template_alias
    )

    if template is None:
        logger.error(f"Template not found by [{state_oj.oj},{state_oj.template_alias}]")
        return False

    source_file_name = os.path.basename(template.path)

    if len(in_out_list) == 0:
        testcase_folder = os.path.abspath(".")
        assert IN_PREFIX != OUT_PREFIX
        files = set(os.listdir(os.path.join(testcase_folder)))
        for filename in files:
            if filename.startswith(IN_PREFIX):
                out_name = OUT_PREFIX + filename[len(IN_PREFIX) :]
                if out_name in files:
                    in_out_list.append(
                        (
                            os.path.join(testcase_folder, filename),
                            os.path.join(testcase_folder, out_name),
                        )
                    )
                else:
                    logger.warn(
                        f"[red]Input file found <{filename}>, output file not found.[/red]"
                    )
                    console.print(
                        f"[red]Input file found <{filename}>, output file not found.[/red]"
                    )

    os.makedirs(TEST_FOLDER, exist_ok=True)
    print(in_out_list)
    tester(
        in_out_list=in_out_list,
        work_folder=os.path.abspath(TEST_FOLDER),
        test_files=[os.path.abspath(source_file_name)],
        template=template,
        diff_fn=diff_result_fn,
    )

    # shutil.rmtree(TEST_FOLDER)
    # logger.debug('test finished')
    return True


@click.command(name="test")
@click.option("-i", "--input", default="")
@click.option("-o", "--output", default="")
def t3st_command(input: str, output: str) -> None:
    try:
        if (input != "") == (output != ""):
            if input != "":
                t3st_main([(os.path.abspath(input), os.path.abspath(output))])
            else:
                t3st_main([])
        else:
            console.print("[red] input and output should be both provided[/red]")
    except KeyboardInterrupt:
        logger.info("Interrupt by user")
    except Exception:
        logger.error(traceback.format_exc())



import logging
import click
from requests import ConnectTimeout, ReadTimeout
from rich.console import Console
from rich.table import Table

from oi_cli2.cli.adaptor.ojman import OJManager
from oi_cli2.core.DI import DI_ACCMAN, DI_LOGGER
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.utils.Provider2 import Provider2
from oi_cli2.utils.account import AccountManager

console = Console(color_system='256', style=None)


@click.command(name="lang")
@click.argument('platform')
def lang_command(platform) -> None:
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  try:
    languages = oj.get_language()
  except (ReadTimeout, ConnectTimeout) as e:
    logger.error(f'Http Timeout[{type(e).__name__}]: {e.request.url}')
    return
  except Exception as e:
    logger.exception(e)
    return

  table = Table().grid()
  table.add_column('Lang')
  table.add_column('value')
  for k, v in languages.items():
    table.add_row(v, k)
  console.print(table)



from dataclasses import dataclass, field
from typing import List, Optional

from rich.console import Console
from rich.table import Table
from rich.style import Style

# TODO support different type table col
# https://codeforces.com/contest/1633/standings
from codeforces_core.contest_standing import Standing


@dataclass
class StandingProblem:
  id: str = ''
  score: str = ''
  time: str = ''


@dataclass
class StandingRow:
  rank: str = ''
  who: str = ''
  passed: Optional[str] = None
  score: str = ''
  hack: str = ''
  penalty: str = ''
  problems: List[StandingProblem] = field(default_factory=lambda: [])


def printData(result: Standing, title: str, handle: str):
  # print(res.text)
  head = result.head
  rows = result.rows
  console = Console()

  table = Table(title=title)
  for h in head:
    if h == 'rank':
      table.add_column(h, style="cyan")
    else:
      table.add_column(h)

  for item in rows:
    row: List[str] = []
    for i in range(len(head)):
      # TODO reconstruct
      if head[i] == "rank":
        row.append(item.rank)
      elif head[i] == "who":
        row.append(item.who)
      elif head[i] == "hack":
        row.append(item.hack)
      elif head[i] == "score":
        row.append(item.score)
      elif head[i] == "penalty":
        row.append(item.penalty)

    for p in item.problems:
      if p.time:
        row.append(f"{p.score}({p.time})")
      else:
        row.append(p.score)

    table.add_row(*row, style=Style(bgcolor="dark_green" if item.who == handle or item.who == f'* {handle}' else None))

  console.print(table)



from typing import Callable, Dict
from oi_cli2.cli.adaptor.AtCoderAdaptor import AtCoder, AtcoderGen
from oi_cli2.cli.adaptor.Codeforces.CodeforcesAdaptor import CodeforcesGen
from oi_cli2.cli.adaptor.Codeforces.Codeforces import Codeforces
from oi_cli2.model.Account import Account
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.utils.Provider2 import Provider2

reg_list: Dict[str, Callable[[Account, Provider2], BaseOj]] = {
    Codeforces.__name__: CodeforcesGen,
    AtCoder.__name__: AtcoderGen,
}



import asyncio
import logging
import os
from typing import Any, Tuple, AsyncIterator

from requests.exceptions import ReadTimeout, ConnectTimeout

from oi_cli2.cli.constant import APP_NAME, CIPHER_KEY, GREEN, DEFAULT
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.model.ParseProblemResult import ParsedProblemResult
from oi_cli2.model.LangKV import LangKV
from oi_cli2.model.Account import Account
from oi_cli2.model.ProblemMeta import ContestMeta, ProblemMeta, E_STATUS
from oi_cli2.model.Result import SubmissionResult
from oi_cli2.model.TestCase import TestCase
# from oi_cli2.utils.async2sync import iter_over_async
from oi_cli2.utils.enc import AESCipher

from codeforces_core.account import async_login, async_fetch_logged_in
from codeforces_core.contest_list import async_contest_list
from codeforces_core.contest_register import async_register, RegisterResultMsg
from codeforces_core.contest_standing import async_friends_standing
from codeforces_core.contest_meta import async_contest_meta, ProblemMeta as InnerProblemMeta
from codeforces_core.interfaces.AioHttpHelper import AioHttpHelperInterface
from codeforces_core.language import async_language
from codeforces_core.problem import async_problem
from codeforces_core.submit import async_submit, transform_submission, async_fetch_submission_page, SubmissionWSResult, SubmissionPageResult
from codeforces_core.url import pid2url, pid2split, problem_url_parse
from codeforces_core.websocket import create_contest_ws_task_yield


class Codeforces(BaseOj):

  def __init__(self, http_util: AioHttpHelperInterface, logger: logging.Logger, account: Account) -> None:
    super().__init__()
    assert (account is not None)
    self._base_url = 'https://codeforces.com/'
    self.logger: logging.Logger = logger
    self.account: Account = account
    self.http = http_util
    self.api_sub_logger: logging.Logger = logging.getLogger(f'{APP_NAME}.yxr-cf-core')

  async def init(self) -> None:
    await self.http.open_session()

  async def deinit(self) -> None:
    await self.http.close_session()

  def pid2url(self, problem_id: str):
    return self._base_url[:-1] + pid2url(problem_id)

  def pid2file_path(self, problem_id: str):
    contest_id, problem_key = pid2split(problem_id)
    return os.path.join(contest_id, problem_key)

  def problem_by_id(self, problem_id: str) -> ParsedProblemResult:
    return self.async_2_sync_session_wrap(lambda: self.async_problem_by_id(problem_id))

  async def async_problem_by_id(self, problem_id: str) -> ParsedProblemResult:
    contest_id, problem_key = pid2split(problem_id)
    self.logger.debug(f'{problem_id} => {contest_id}, {problem_key}')

    result = await async_problem(http=self.http, contest_id=contest_id, level=problem_key)

    return ParsedProblemResult(
        status=ParsedProblemResult.Status.NOTVIS,  # TODO for show progress
        title=result.title,
        test_cases=list(map(lambda x: TestCase(in_data=x.in_data, out_data=x.out_data), result.test_cases)),
        id=problem_id,
        oj=Codeforces.__name__,
        description=result.description,
        time_limit=result.time_limit,
        mem_limit=result.mem_limit,
        url=self.pid2url(problem_id),
        html=result.html,
        file_path=self.pid2file_path(problem_id))

  def problem(self, problem: ProblemMeta) -> ParsedProblemResult:
    return self.problem_by_id(problem.contest_id + problem.id)

  async def async_problem(self, problem: ProblemMeta) -> ParsedProblemResult:
    return await self.async_problem_by_id(problem.contest_id + problem.id)

  def login_website(self, force=False) -> bool:  # return successful
    return self.async_2_sync_session_wrap(lambda: self.async_login_website(force=force))

  # Force: true/false login whatever login before
  # TODO 逻辑还是有点问题，要实现支持
  #  - 未登录 => 登录
  #  - 已登录 => 不操作
  #  强制:
  #   - 未登录 => 登录
  #   - 已登录 => 取消cookies等 强制登录
  async def async_login_website(self, force=False) -> bool:  # return successful
    if not force:
      # try using cookies
      self.logger.info(f"{GREEN}Checking Log in {DEFAULT}")
      try:
        if await self.async_is_login():
          self.logger.info(f"{GREEN}{self.account.account} is Logged in {Codeforces.__name__}{DEFAULT}")
          return True
      except (ReadTimeout, ConnectTimeout) as e:
        self.logger.error(f'Http Timeout[{type(e).__name__}]: {e.request.url}')
      except Exception as e:
        self.logger.exception(e)

    try:
      self.logger.debug(f"{GREEN}{self.account.account} Logining {Codeforces.__name__}{DEFAULT}")

      return (await async_login(http=self.http,
                                handle=self.account.account,
                                password=AESCipher(CIPHER_KEY).decrypt(self.account.password))).success
    except (ReadTimeout, ConnectTimeout) as e:
      self.logger.error(f'Http Timeout[{type(e).__name__}]: {e.request.url}')
    except Exception as e:
      self.logger.exception(e)

    return False

  def _is_login(self) -> bool:
    return self.async_2_sync_session_wrap(lambda: self.async_is_login())

  async def async_is_login(self) -> bool:
    ok, html_data = await async_fetch_logged_in(self.http)
    return ok

  def reg_contest(self, contest_id: str) -> bool:
    return self.async_2_sync_session_wrap(lambda: self.async_reg_contest(contest_id))

  async def async_reg_contest(self, contest_id: str) -> bool:
    if not await self.async_login_website():
      raise Exception('Login Failed')
    result = await async_register(http=self.http, contest_id=contest_id)
    return result.msg == RegisterResultMsg.AlreadyRegistered or result.msg == RegisterResultMsg.HaveBeenRegistered

  def submit_code(self, problem_url: str, language_id: str, code_path: str) -> bool:
    # https://codeforces.com/contest/1740/problem/G
    contest_id, problem_key = problem_url_parse(problem_url)
    sid = contest_id + problem_key
    return self.async_2_sync_session_wrap(lambda: self.async_submit_code(sid, language_id, code_path))

  # TODO move sid out as just Syntactic sugar
  async def async_submit_code(self, sid: str, language_id: str, code_path: str) -> bool:
    if not await self.async_login_website():
      raise Exception('Login Failed')
    contest_id, problem_key = pid2split(sid)
    self.logger.debug(f'{contest_id},{problem_key}')

    submit_id, resp = await async_submit(http=self.http,
                                         contest_id=contest_id,
                                         level=problem_key,
                                         file_path=code_path,
                                         lang_id=language_id,
                                         logger=self.api_sub_logger)
    self.logger.debug(f'submit_id = {submit_id}')
    return bool(submit_id)

  async def async_get_result_yield(self, problem_url: str, time_gap: float = 2) -> AsyncIterator[SubmissionResult]:
    contest_id, problem_key = problem_url_parse(problem_url)

    # TODO move more parse inside codeforces-core ? cf是出错中断形式,状态+数量
    def page_result_transform(res: SubmissionPageResult) -> SubmissionResult:
      self.logger.debug('page res:' + str(res))
      cur_status = SubmissionResult.Status.PENDING
      if res.verdict.startswith('Running'):
        cur_status = SubmissionResult.Status.RUNNING
      elif res.verdict.startswith('In queue'):
        cur_status = SubmissionResult.Status.RUNNING
      elif res.verdict.startswith('Accepted'):
        cur_status = SubmissionResult.Status.AC
      elif res.verdict.startswith('Pretests passed'):
        cur_status = SubmissionResult.Status.AC
      elif res.verdict.startswith('Wrong answer'):
        cur_status = SubmissionResult.Status.WA
      elif res.verdict.startswith('Time limit exceeded'):
        cur_status = SubmissionResult.Status.TLE
      elif res.verdict.startswith('Runtime error'):  # Runtime error on pretest 2
        cur_status = SubmissionResult.Status.RE
      else:
        self.logger.error('NOT HANDLE PAGE:' + str(res.verdict))

      if res.url.startswith('/'):
        res.url = self._base_url[:-1] + res.url

      return SubmissionResult(id=res.id,
                              cur_status=cur_status,
                              time_note=res.time_ms + ' ms',
                              mem_note=str(int(res.mem_bytes) / 1000) + ' kb',
                              url=res.url,
                              msg_txt=res.verdict)

    # TODO move more parse inside codeforces-core ?
    def ws_result_transform(res: SubmissionWSResult) -> SubmissionResult:
      cur_status = SubmissionResult.Status.PENDING
      if res.msg == 'TESTING':
        cur_status = SubmissionResult.Status.RUNNING
      elif res.msg == 'OK':
        cur_status = SubmissionResult.Status.AC
      elif res.msg == 'WRONG_ANSWER':
        cur_status = SubmissionResult.Status.WA
      else:
        self.logger.error('NOT HANDLE WS:' + str(res.msg))

      msg_txt = str(res.testcases)
      if cur_status in [SubmissionResult.Status.AC, SubmissionResult.Status.WA]:
        msg_txt = f'{res.passed}/{res.testcases}'

      return SubmissionResult(
          id=str(res.submit_id),
          cur_status=cur_status,
          time_note=str(res.ms) + ' ms',
          mem_note=str(int(res.mem) / 1000) + ' kb',
          url=f'{self._base_url}contest/{res.contest_id}/submission/{res.submit_id}',
          msg_txt=msg_txt,
      )

    # TODO visit page without ws first
    results = await async_fetch_submission_page(http=self.http, problem_url=problem_url, logger=self.api_sub_logger)
    fix_submit_id = ''
    if len(results) > 0:
      result = page_result_transform(results[0])
      fix_submit_id = result.id
      self.logger.debug(f"fix submit_id = {fix_submit_id}")
      yield result
      if result.cur_status not in [SubmissionResult.Status.PENDING, SubmissionResult.Status.RUNNING]:
        return

    self.logger.debug('after page result, enter ws result')

    # return (end watch?, transform result)
    def custom_handler(result: Any) -> Tuple[bool, SubmissionWSResult]:
      parsed_data = transform_submission(result)
      if fix_submit_id and fix_submit_id != parsed_data.contest_id:  # submit id not match, dont end watch ws
        return False, parsed_data
      if parsed_data.msg != 'TESTING':
        return True, parsed_data
      return False, parsed_data

    # TODO add timeout for ws
    # TODO 可能有别人的? pc/cc?
    async for wsresult in create_contest_ws_task_yield(http=self.http,
                                                       contest_id=contest_id,
                                                       ws_handler=custom_handler,
                                                       logger=self.api_sub_logger):
      self.logger.debug('ws res:' + str(wsresult))
      if fix_submit_id and str(wsresult.submit_id) != fix_submit_id:
        self.logger.debug(f'[skip]fixed id not match! continue, fixed[{fix_submit_id}], ws[{wsresult.submit_id}]]')
        continue
      data = ws_result_transform(wsresult)
      yield data
      if data.cur_status not in [SubmissionResult.Status.PENDING, SubmissionResult.Status.RUNNING]:
        return

    results = await async_fetch_submission_page(http=self.http, problem_url=problem_url)
    assert len(results) > 0
    yield page_result_transform(results[0])

  def get_language(self) -> LangKV:
    return self.async_2_sync_session_wrap(lambda: self.async_get_language())

  async def async_get_language(self) -> LangKV:
    await self.async_login_website()
    res = await async_language(self.http)
    ret: LangKV = {}
    for item in res:
      ret[item.value] = item.text
    return ret

  @staticmethod
  def support_contest() -> bool:
    return True

  def print_contest_list(self) -> bool:
    return self.async_2_sync_session_wrap(lambda: self.async_print_contest_list())

  async def async_print_contest_list(self) -> bool:
    await self.async_login_website()

    result = await async_contest_list(http=self.http)
    from .contestList import printData
    printData(result)
    return True

  def get_contest_meta(self, contest_id: str) -> ContestMeta:
    return self.async_2_sync_session_wrap(lambda: self.async_get_contest_meta(contest_id=contest_id))

  async def async_get_contest_meta(self, contest_id: str) -> ContestMeta:
    await self.async_login_website()

    result = await async_contest_meta(http=self.http, contest_id=contest_id)

    def transform(problem: InnerProblemMeta) -> ProblemMeta:
      return ProblemMeta(
          id=problem.id,
          url=problem.url,
          name=problem.name,
          passed=problem.passed,  # number of passed submission in contest
          score=0,
          status=E_STATUS(problem.status),  # ???? TODO
          time_limit_msec=problem.time_limit_msec,  # ms
          memory_limit_kb=problem.memory_limit_kb,  # mb
          contest_id=problem.contest_id,
      )

    return ContestMeta(id=contest_id, url=result.url, problems=list(map(lambda o: transform(o), result.problems)))

  def async_2_sync_session_wrap(self, fn):

    async def task():
      await self.http.open_session()
      result = await fn()
      await self.http.close_session()
      return result

    return asyncio.run(task())

  def print_friends_standing(self, cid: str) -> None:
    return self.async_2_sync_session_wrap(lambda: self.async_print_friends_standing(cid))

  async def async_print_friends_standing(self, cid: str) -> None:
    result = await async_friends_standing(http=self.http, contest_id=cid)
    from .standing import printData
    printData(result, title=f"Friends standing {result.url}", handle=self.account.account)

  def cid2url(self, cid: str) -> str:
    return f'{self._base_url}contests/{cid}'



class Template:
  platform: str
  alias: str
  # relative
  path: str
  compilation: str
  execute: str
  clean: str
  default: bool
  uplang: str

  def __init__(self):
    self.platform = ''
    self.alias = ''
    self.path = ''
    self.compilation = ''
    self.execute = ''
    self.clean = ''  # deperated
    self.uplang = ''
    self.default = False

  # TODO 改成非 class 的 struct
  def initial(self, platform, alias, path, compilation, execute, uplang, default=False):
    self.platform = platform
    self.alias = alias
    self.path = path
    self.compilation = compilation
    self.execute = execute
    # self.clean = clean
    self.uplang = uplang
    self.default = default
    return self

  def dict_init(self, d):
    for key in ['platform', 'alias', 'path', 'compilation', 'execute', 'uplang']:
      if key not in d:
        d[key] = ''
    self.__dict__ = d
    if not hasattr(self, 'default'):
      self.default = False
    return self

  def __repr__(self):
    return "<Template alias:%s path:%s>" % (self.alias, self.path)

  def __str__(self):
    return f"Template({self.alias},{self.platform},{self.compilation})"



#!/usr/bin/env python3
import asyncio
from enum import Enum
import json
import logging
import os
import subprocess
import sys
import threading
import time
from requests.exceptions import ReadTimeout, ConnectTimeout
from typing import List
import click
from rich.live import Live
from rich.table import Table
from rich.style import Style
from rich.console import Console
from oi_cli2.cli.adaptor.ojman import OJManager
from oi_cli2.core.DI import DI_ACCMAN, DI_CFG, DI_LOGGER, DI_TEMPMAN

from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.model.FolderState import FolderState
from oi_cli2.model.ProblemMeta import E_STATUS, ProblemMeta
from oi_cli2.model.TestCase import TestCase

from oi_cli2.utils.FileUtil import FileUtil
from oi_cli2.utils.Provider2 import Provider2
from oi_cli2.utils.account import AccountManager
from oi_cli2.utils.configFolder import ConfigFolder

# file_util can be any thing , everything is file
from oi_cli2.utils.template import TemplateManager

console = Console(color_system='256', style=None)


class VisitStatus(Enum):
  BEFORE = 1
  SUCCESS = 2
  FAILED = 3
  TIMEOUT = 3


visitedTextMap = {
    VisitStatus.BEFORE: "Fetching",
    VisitStatus.SUCCESS: "[green]OK",
    VisitStatus.FAILED: "[red]Failed",
    VisitStatus.TIMEOUT: "[red]Timeout",
}


def all_visit(result) -> bool:
  for v in result:
    if v[1] == VisitStatus.BEFORE:
      return False
  return True


def generate_table(result) -> Table:
  """Make a new table."""
  table = Table()
  table.add_column("Problem", width=10)
  table.add_column("Fetched", width=10)
  table.add_column("Parse Status")

  for row in result:
    table.add_row(
        row[0],
        visitedTextMap[row[1]],
        "[green]OK" if row[2] else "Unknown",
    )
  return table


# TODO support by problem id
async def create_problem(data, pm: ProblemMeta, contest_id: str, template, oj: BaseOj):
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  config_folder: ConfigFolder = Provider2().get(DI_CFG)
  timeOutRetry = 3
  while timeOutRetry >= 0:
    try:
      file_util = FileUtil
      problem_id = contest_id + pm.id
      result = await oj.async_problem(pm)
      data[1] = VisitStatus.SUCCESS
      test_cases: List[TestCase] = result.test_cases
      directory = config_folder.get_file_path(os.path.join('dist', type(oj).__name__, contest_id, pm.id))

      for i in range(len(test_cases)):
        file_util.write(config_folder.get_file_path(os.path.join(directory, f'in.{i}')), test_cases[i].in_data)
        file_util.write(config_folder.get_file_path(os.path.join(directory, f'out.{i}')), test_cases[i].out_data)

      # if code file exist not cover code
      if not os.path.exists(config_folder.get_file_path(os.path.join(directory, os.path.basename(template.path)))):
        file_util.copy(config_folder.get_file_path(template.path),
                       config_folder.get_file_path(os.path.join(directory, os.path.basename(template.path))))
      # TODO 生成state.json ( 提供 自定义字段)
      STATE_FILE = 'state.json'

      # TODO provide more info, like single test and
      # generate state.json
      folder_state = FolderState(oj=type(oj).__name__,
                                 cid=contest_id,
                                 pid=pm.id,
                                 sid=problem_id,
                                 problem_url=pm.url,
                                 template_alias=template.alias,
                                 up_lang=template.uplang)  # TODO get data from analyzer
      logger.debug(f'create folder_state {folder_state}')
      with open(config_folder.get_file_path(os.path.join(directory, STATE_FILE)), "w") as statejson:
        json.dump(folder_state.__dict__, statejson)
        statejson.close()
      data[2] = True
      return True
    except (ReadTimeout, ConnectTimeout) as e:
      logger.info(f'Http Timeout[{type(e).__name__}]: {e.request.url}')
      if timeOutRetry == 0:
        data[1] = VisitStatus.TIMEOUT
        return False
      timeOutRetry -= 1
      time.sleep(0.5)
    except Exception as e:
      logger.exception(type(e).__name__)
      logger.exception(e)
      data[1] = VisitStatus.FAILED
      return False
  return False


async def createDir(oj: BaseOj, contest_id: str, problems: List[ProblemMeta]):
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  config_folder: ConfigFolder = Provider2().get(DI_CFG)
  template_manager: TemplateManager = Provider2().get(DI_TEMPMAN)
  template = template_manager.get_platform_default(type(oj).__name__)
  if template is None:
    logger.error(type(oj).__name__ + ' has no default template, run `oi config template` first')
    return None

  async def sync_oj():  # use thread
    # ID, Fetched, success
    result = []
    for v in problems:
      result.append([v.id, VisitStatus.BEFORE, False])

    def between_callback(*args):
      loop = asyncio.new_event_loop()
      asyncio.set_event_loop(loop)

      loop.run_until_complete(create_problem(*args))
      loop.close()

    tasks = []
    for i in range(len(problems)):
      task = threading.Thread(target=between_callback, args=(result[i], problems[i], contest_id, template, oj))
      tasks.append(task)
      task.start()

    with Live(generate_table(result), auto_refresh=False) as live:
      while not all_visit(result):
        await asyncio.sleep(0.05)  # time.sleep 会卡住
        live.update(generate_table(result), refresh=True)

    for t in tasks:  # wait all task finished
      t.join()

    live.update(generate_table(result), refresh=True)

    return config_folder.get_file_path(os.path.join('dist', type(oj).__name__, contest_id))

  async def async_oj():  # use async/await
    # ID, Fetched, success
    result = []
    for v in problems:
      result.append([v.id, VisitStatus.BEFORE, False])

    tasks = []
    for i in range(len(problems)):
      tasks.append(asyncio.create_task(create_problem(result[i], problems[i], contest_id, template, oj)))

    with Live(generate_table(result), auto_refresh=False) as live:
      while not all_visit(result):
        await asyncio.sleep(0.05)  # time.sleep 会卡住
        live.update(generate_table(result), refresh=True)

    asyncio.gather(*tasks)  # wait all task finished

    live.update(generate_table(result), refresh=True)
    return config_folder.get_file_path(os.path.join('dist', type(oj).__name__, contest_id))

  if oj.__class__.__name__ == 'AtCoder':
    return await sync_oj()
  elif oj.__class__.__name__ == 'Codeforces':
    return await async_oj()
  else:
    raise Exception(f'Un specific oj[{oj.__class__.__name__}] sync/async type')


@click.group()
def contest():
  """Contest relative(fetch,list,standing,detail)"""


@contest.command()
@click.argument('platform')
@click.argument('contestid')
def fetch(platform, contestid) -> None:
  asyncio.run(async_fetch(platform, contestid))


async def async_fetch(platform, contestid) -> None:
  """Fetch a contest(problems and testcases)

  PLATFORM    e.g. AtCoder, Codeforces

  CONTESTID   The id in the url, e.g. Codeforces(1122),AtCoder(abc230)
  """
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  await oj.init()
  try:
    problems = (await oj.async_get_contest_meta(contestid)).problems
  except (ReadTimeout, ConnectTimeout) as e:
    logger.error(f'Http Timeout[{type(e).__name__}]: {e.request.url}')
    return
  except Exception as e:
    logger.exception(e)
    return

  try:
    logger.debug(f"{contestid},{problems}")
    directory = await createDir(oj=oj, contest_id=contestid, problems=problems)
    console.print(f"[green bold] {directory} ")
  except Exception as e:
    logger.error(e)
  await oj.deinit()


@contest.command(name='list')
@click.argument('platform')
def list_command(platform: str):
  """list passed and recent contest

  PLATFORM    e.g. AtCoder, Codeforces
  """
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  oj.print_contest_list()


@contest.command()
@click.argument('platform')
@click.argument('contestid')
def detail(platform, contestid) -> None:
  """Display problem set and your submit status of a specific contest"

  PLATFORM    e.g. AtCoder, Codeforces

  CONTESTID   The id in the url, e.g. Codeforces(1122),AtCoder(abc230)
  """
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  cm = oj.get_contest_meta(contestid)
  table = Table(title=f"Contest {cm.url}")
  table.add_column("ID", style="cyan", no_wrap=False)
  table.add_column("Name")
  table.add_column("Time")
  table.add_column("Memory")
  table.add_column("Passed")
  table.add_column("Url")
  for o in cm.problems:
    style = Style()

    if o.status == E_STATUS.AC:
      style = Style(bgcolor="dark_green")
    elif o.status == E_STATUS.ERROR:
      style = Style(bgcolor="dark_red")
    table.add_row(o.id,
                  o.name,
                  str(o.time_limit_msec / 1000) + "s",
                  str(o.memory_limit_kb / 1000) + "mb",
                  o.passed,
                  o.url,
                  style=style)

  console = Console()
  console.print(table)


@contest.command()
@click.argument('platform')
@click.argument('contestid')
def standing(platform, contestid) -> None:
  """Display your friends standing of a specific contest"

  PLATFORM    e.g. AtCoder, Codeforces

  CONTESTID   The id in the url, e.g. Codeforces(1122),AtCoder(abc230)
  """
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  oj.print_friends_standing(cid=contestid)


@contest.command(name='open')
@click.argument('platform')
@click.argument('contestid')
def open_browser(platform, contestid) -> None:

  def open_url_with_default_browser(url: str) -> None:
    if sys.platform == 'win32':
      os.startfile(url)
    elif sys.platform == 'darwin':
      subprocess.Popen(['open', url])
    else:
      try:
        subprocess.Popen(['xdg-open', url])
      except OSError:
        print("Please open a browser on: " + url)

  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  open_url_with_default_browser(oj.cid2url(contestid))


@contest.command()
@click.argument('platform')
@click.argument('contestid')
def reg(platform, contestid) -> None:
  logger: logging.Logger = Provider2().get(DI_LOGGER)
  am: AccountManager = Provider2().get(DI_ACCMAN)

  try:
    oj: BaseOj = OJManager.createOj(platform=platform,
                                    account=am.get_default_account(platform=platform),
                                    provider=Provider2())
  except Exception as e:
    logger.exception(e)
    raise e

  oj.reg_contest(contestid)



from enum import Enum
from bs4 import element

from oi_cli2.utils.HttpUtil import HttpUtil


class HtmlTagAbstract(object):

  def __init__(self, http_util: HttpUtil) -> None:
    self.http_util = http_util

  class TagDesc(Enum):
    """
        给html的tag加上相应的class

        TITLE = 'vj-title'
        CONTENT = 'vj-content'
        IMAGE = 'vj-image'
        FILE = 'vj-file'
        ANCHOR = 'vj-anchor'

        """
    TITLE = 'vj-title'
    CONTENT = 'vj-content'
    IMAGE = 'vj-image'
    FILE = 'vj-file'
    ANCHOR = 'vj-anchor'

  class TagStyle(Enum):
    """
        TITLE 和 CONTENT 需要加额外的 Style 保证网页风格一致
        """
    TITLE = 'font-family: "Helvetica Neue",Helvetica,"PingFang SC","Hiragino Sans GB"' \
            ',"Microsoft YaHei","微软雅黑",Arial,sans-serif; font-size: 16px;font-weight: bold;color:#000000;'
    CONTENT = 'font-family: "Helvetica Neue",Helvetica,"PingFang SC","Hiragino Sans GB",' \
              '"Microsoft YaHei","微软雅黑",Arial,sans-serif; font-size: 16px;color:#495060;'



from datetime import datetime, timedelta
import time


def moscow_to_utc(dt: datetime) -> datetime:
  return dt - timedelta(hours=3)


def utc_to_local(utc_datetime: datetime) -> datetime:
  now_timestamp = time.time()
  offset = datetime.fromtimestamp(now_timestamp) - datetime.utcfromtimestamp(now_timestamp)
  return utc_datetime + offset



from typing import List

from oi_cli2.model.ParseProblemResult import ParsedProblemResult
from oi_cli2.model.TestCase import TestCase
from oi_cli2.utils.FileUtil import FileUtil


class problem:

  def __init__(self, oj):
    self.oj = oj

  # file_util can be any thing , everything is file
  def parse(self, problem_id: str, file_util: FileUtil):
    result = self.oj.parse(problem_id)
    # html: str = result.html
    test_cases: List[TestCase] = result.test_cases

    # TODO switch directory

    for i in range(len(test_cases)):
      file_util.write(f'in.{i}', test_cases[i].in_data)
      file_util.write(f'out.{i}', test_cases[i].in_data)



import os
from shutil import copyfile


class FileUtil:

  @staticmethod
  def write(out_path: str, data: str):
    # if no folder create folder

    directory = os.path.dirname(out_path)
    if not os.path.exists(directory):
      os.makedirs(directory)
    with open(out_path, "w") as file_ins:
      file_ins.write(data)
      file_ins.close()

  @staticmethod
  def copy(source_path: str, dest_path: str):
    directory = os.path.dirname(dest_path)
    if not os.path.exists(directory):
      os.makedirs(directory)
    copyfile(source_path, dest_path)

  @staticmethod
  def read(out_path: str) -> str:
    assert False
    return f'TODO {out_path}'



import os
import sys
import click
from oi_cli2.cli.constant import OT_FOLDER, USER_CONFIG_FILE
from oi_cli2.utils.configFolder import ConfigFolder


@click.command()
def init():
  """Init oi folder"""
  folder = OT_FOLDER
  config_folder = ConfigFolder(folder).get_root_folder()
  if config_folder is None:
    os.makedirs(folder)
    print(folder + ' Created, oi folder inited.')
    with open(".gitignore", 'a') as f:
      f.write(os.path.join(folder, USER_CONFIG_FILE))
  else:
    print(config_folder + '/.oiTerminal Exist', file=sys.stderr)



class Analyze:
  # TODO platform
  platform: str
  submit_lang: str
  template_alias: str
  class_path: str  # 实例class 文件
  default: bool

  def __init__(self):
    self.platform = ''
    self.submit_lang = ''
    self.template_alias = ''
    self.class_path = ''  # 实例class 文件
    self.default = False

  def initial(self, platform, submit_lang, template_alias, class_path, default=False):
    self.platform = platform
    self.submit_lang = submit_lang
    self.template_alias = template_alias
    self.class_path = class_path  # 实例class 文件
    self.default = default
    return self

  def dict_init(self, d):
    for key in ['platform', 'submit_lang', 'template_alias', 'class_path']:
      if key not in d:
        d[key] = ''
    self.__dict__ = d
    if not hasattr(self, 'default'):
      self.default = False
    return self

  def __repr__(self):
    return "<Analyze platform:%s template:%s>" % (self.platform, self.template_alias)

  def __str__(self):
    return f"Analyze({self.platform},{self.template_alias})"



from typing import List, Optional
import logging

from oi_cli2.model.Template import Template
from oi_cli2.utils.db import JsonFileDB
from oi_cli2.utils.consts.ids import Ids


class TemplateManager:

  def __init__(self, db: JsonFileDB, platform: str = ''):
    self.db = db
    self.platform = platform
    self.keys = ['platform', 'alias', 'path', 'compilation', 'execute', 'clean', 'default']

  def _get_template_list(self) -> List[Template]:
    temp_list: List[dict] = self.db.load(Ids.template) or []
    return list(map(lambda d: Template().dict_init(d), temp_list))

  def _set_template_list(self, temp_list: List[Template]):
    temp_list.sort(key=lambda temp0: (temp0.platform, -temp0.default, temp0.alias))
    self.db.save(Ids.template, list(map(lambda d: d.__dict__, temp_list)))

  def get_list(self) -> List[Template]:
    return self._get_template_list()

  def alias_exist(self, temps: List[Template], platform: str, alias: str):
    return self.find_alias(temps=temps, platform=platform, alias=alias) != -1

  def find_alias(self, temps: List[Template], platform: str, alias: str) -> int:
    for i in range(len(temps)):
      if temps[i].platform == platform and temps[i].alias == alias:
        return i
    return -1

  def get_platform_default(self, platform: str) -> Optional[Template]:
    temps: List[Template] = self._get_template_list()
    for i in range(len(temps)):
      if temps[i].platform == platform and temps[i].default:
        return temps[i]
    return None

  def get_default(self) -> Optional[Template]:
    if not self.platform:
      logging.error('Please set platform first or using get_platform_default()')
      return None
    temps: List[Template] = self._get_template_list()
    for i in range(len(temps)):
      if temps[i].platform == self.platform and temps[i].default:
        return temps[i]
    return None

  def get_template_by_name(self, platform: str, name: str) -> Optional[Template]:
    temps: List[Template] = self._get_template_list()
    for i in range(len(temps)):
      if temps[i].platform == platform and temps[i].alias == name:
        return temps[i]
    return None

  def get_template_by_alias(self, alias: str) -> Optional[Template]:
    # deperated
    assert False
    pass
    if not self.platform:
      logging.error('Please set platform first or using get_platform_default()')
      return None
    temps: List[Template] = self._get_template_list()
    for i in range(len(temps)):
      if temps[i].platform == self.platform and temps[i].alias == alias:
        return temps[i]
    return None

  def set_temps_default(self, temps: List[Template], index: int):
    assert 0 <= index < len(temps)
    for i in range(len(temps)):
      if i == index:
        temps[i].default = True
      elif temps[i].platform == temps[index].platform:
        temps[i].default = False

  def set_default(self, index: int):
    temps: List[Template] = self._get_template_list()
    assert 0 <= index < len(temps)
    for i in range(len(temps)):
      if i == index:
        temps[i].default = True
      elif temps[i].platform == temps[index].platform:
        temps[i].default = False

    self._set_template_list(temps)

  def delete_template(self, platform: str, name: str) -> bool:
    temps: List[Template] = self._get_template_list()
    idx = -1
    for i in range(len(temps)):
      if temps[i].platform == platform and temps[i].alias == name:
        idx = i
        break
    if idx < 0:
      return False

    if temps[idx].default:
      for i in range(len(temps)):
        if i == idx:
          continue
        if temps[i].platform == temps[idx].platform:
          temps[i].default = True
          break

    del temps[idx]
    self._set_template_list(temps)
    return True

  # set default if no platform there
  def add_template(self, platform, alias, path, compilation, execute, uplang) -> None:
    temps: List[Template] = self._get_template_list()
    if self.find_alias(temps, platform, alias) != -1:
      raise Exception('Duplicate alias')

    is_default = True
    for item in temps:
      if item.platform == platform and item.default:
        is_default = False
        break

    temps.append(Template().initial(platform=platform,
                                    alias=alias,
                                    path=path,
                                    compilation=compilation,
                                    execute=execute,
                                    uplang=uplang,
                                    default=is_default))
    self._set_template_list(temps)

  def modify_alias(self, index: int, value: str):
    temps: List[Template] = self._get_template_list()
    assert 0 <= index < len(temps)
    if self.alias_exist(temps, temps[index].platform, value):
      raise Exception('Duplicate alias')
    temps[index].alias = value
    self._set_template_list(temps)

  def modify_path(self, index: int, value: str):
    temps: List[Template] = self._get_template_list()
    assert 0 <= index < len(temps)
    temps[index].path = value
    self._set_template_list(temps)

  def modify_compilation(self, index: int, value: str):
    temps: List[Template] = self._get_template_list()
    assert 0 <= index < len(temps)
    temps[index].compilation = value
    self._set_template_list(temps)

  def modify_execute(self, index: int, value: str):
    temps: List[Template] = self._get_template_list()
    assert 0 <= index < len(temps)
    temps[index].execute = value
    self._set_template_list(temps)

  def modify_clean(self, index: int, value: str):
    temps: List[Template] = self._get_template_list()
    assert 0 <= index < len(temps)
    temps[index].clean = value
    self._set_template_list(temps)

  # update
  def update_template(self, platform, alias: str, newalias: str, path: str, compilation: str, execute: str, uplang: str,
                      default: bool):
    temps: List[Template] = self._get_template_list()
    idx = self.find_alias(temps, platform, alias)
    if idx == -1:
      raise Exception('Template Not Exist')

    if default:
      self.set_temps_default(temps, idx)

    if newalias:
      temps[idx].alias = newalias

    if path:
      temps[idx].path = path

    if compilation:
      temps[idx].compilation = compilation

    if execute:
      temps[idx].execute = execute

    if uplang:
      temps[idx].uplang = uplang

    self._set_template_list(temps)



import logging
import time
from typing import List, AsyncIterator
from rich.console import Console
from rich.table import Table
from rich.style import Style

from oi_cli2.model.LangKV import LangKV

from ...cli.constant import CIPHER_KEY
from ...model.Account import Account
from ...model.BaseOj import BaseOj
from ...model.ParseProblemResult import ParsedProblemResult
from ...model.ProblemMeta import ContestMeta, ProblemMeta
from ...model.Result import SubmissionResult
from ...model.TestCase import TestCase
from ...utils.HtmlTag import HtmlTag
from ...utils.HttpUtil import HttpUtil
from ...utils.HttpUtilCookiesHelper import HttpUtilCookiesHelper
from ...utils.Provider2 import Provider2
from ...utils.enc import AESCipher
from ...abstract.HtmlTagAbstract import HtmlTagAbstract
from ...core.DI import DI_ACCMAN, DI_HTTP, DI_LOGGER, DI_PROVIDER

from ac_core.auth import fetch_login, is_logged_in
from ac_core.contest import fetch_tasks_meta, ParserProblemResult, fetch_standing, fetch_list
from ac_core.problem import parse_task
from ac_core.submit import fetch_submit
from ac_core.interfaces.HttpUtil import HttpRespInterface
from ac_core.result import fetch_result, SubmissionResult as CORE_SUB_RES
from ac_core.language import fetch_language

console = Console(color_system='256', style=None)


def s2str(sec: int) -> str:
  if sec < 60:
    return str(sec)
  if sec < 60 * 60:
    return f"{sec//60}:{(sec%60):02d}"
  return f"{sec // 60 // 60}:{((sec // 60) % 60):02d}:{(sec % 60):02d}"


def transform_Result(res: CORE_SUB_RES) -> SubmissionResult:
  mapdict = {
      CORE_SUB_RES.Status.AC: SubmissionResult.Status.AC,
      CORE_SUB_RES.Status.PENDING: SubmissionResult.Status.PENDING,
      CORE_SUB_RES.Status.RUNNING: SubmissionResult.Status.RUNNING,
      CORE_SUB_RES.Status.INIT: SubmissionResult.Status.PENDING,
      CORE_SUB_RES.Status.RE: SubmissionResult.Status.RE,
      CORE_SUB_RES.Status.TLE: SubmissionResult.Status.TLE,
      CORE_SUB_RES.Status.WA: SubmissionResult.Status.WA,
      CORE_SUB_RES.Status.CE: SubmissionResult.Status.CE,
  }
  if res.status in list(mapdict.keys()):
    status = mapdict[res.status]
  else:
    logger: logging.Logger = Provider2().get(DI_LOGGER)
    logger.error(f'Unknown status {res.status}')
    status = SubmissionResult.Status.UNKNOWN

  return SubmissionResult(
      id=res.id,
      cur_status=status,
      quick_key=res.url,  # for refetch result
      url=res.url,  # TODO change to webpage url
      state_note=str(res.score),
      time_note=str(res.time_cost_ms / 1000) + ' ms',
      mem_note=str(res.mem_cost_kb) + ' kb',
      msg_txt=res.msg_txt,
  )


class AtCoder(BaseOj):

  def __init__(self, http_util: HttpUtil, logger: logging.Logger, account: Account, html_tag: HtmlTagAbstract) -> None:
    super().__init__()
    assert (account is not None)
    self._base_url = 'https://atcoder.jp/'
    self.logger: logging.Logger = logger
    self.html_tag = html_tag
    self.account: Account = account
    self.http_util = http_util
    HttpUtilCookiesHelper.load_cookie(provider=Provider2(), platform=AtCoder.__name__, account=account.account)

  def login_website(self, force: bool = False) -> bool:
    if force or not is_logged_in(self.http_util):  # need login
      if force:
        self.http_util._request.cookies.clear()
      ok = fetch_login(self.http_util, self.account.account, AESCipher(CIPHER_KEY).decrypt(self.account.password))
      # if ok:
      # always save cookie
      HttpUtilCookiesHelper.save_cookie(provider=Provider2(), platform=AtCoder.__name__, account=self.account.account)
      return ok
    return True

  async def async_get_contest_meta(self, cid: str) -> ContestMeta:
    return self.get_contest_meta(cid)

  def get_contest_meta(self, cid: str) -> ContestMeta:
    self.login_website()
    res = fetch_tasks_meta(self.http_util, cid)

    def transform(pm: ParserProblemResult) -> ProblemMeta:
      return ProblemMeta(id=pm.id,
                         url=pm.url,
                         name=pm.name,
                         contest_id=cid,
                         memory_limit_kb=pm.memory_limit_kb,
                         time_limit_msec=pm.time_limit_msec)

    return ContestMeta(id=cid, url=res.url, problems=[transform(pm) for pm in res.problems])

  async def async_problem(self, problem: ProblemMeta) -> ParsedProblemResult:
    return self.problem(problem)

  # Care !! in Atcoder may arc058 C = https://atcoder.jp/contests/arc058/tasks/arc058_a
  def problem(self, pm: ProblemMeta) -> ParsedProblemResult:
    html = self.http_util.get(pm.url).text
    res = parse_task(html=html)
    return ParsedProblemResult(
        # status=: Status = Status.NOTVI STODO
        id=res.id,
        title=pm.name,
        test_cases=[TestCase(in_data=o.input, out_data=o.output) for o in res.tests],
        oj=AtCoder.__name__,
        # description=res.id,
        time_limit=str(pm.time_limit_msec),
        mem_limit=str(pm.memory_limit_kb),
        url=res.url,
    )

  def submit_code(self, problem_url: str, language_id: str, code_path: str) -> HttpRespInterface:
    if not self.login_website():
      raise Exception('Login Failed')

    return fetch_submit(self.http_util,
                        problem_url=problem_url,
                        lang_id=language_id,
                        source_code=open(code_path, 'r').read())

  async def async_get_result_yield(self, problem_url: str, time_gap: float = 1) -> AsyncIterator[SubmissionResult]:
    while True:
      res = transform_Result(fetch_result(self.http_util, problem_url))
      yield res
      if res.cur_status not in [SubmissionResult.Status.PENDING, SubmissionResult.Status.RUNNING]:
        break
      time.sleep(time_gap)

  # TODO fav control ?
  def print_friends_standing(self, cid: str) -> None:
    if not self.login_website():
      raise Exception('Login Failed')

    standing = fetch_standing(self.http_util, contest_id=cid)

    table = Table(title=f"Binary standing {cid}")
    table.add_column("rank", style="cyan")
    table.add_column("handle")
    for task in standing.TaskInfo:
      table.add_column(task.Assignment)

    for i in range(len(standing.StandingsData)):
      row: List[str] = []
      d = standing.StandingsData[i]
      is_self = d.UserName == self.account.account
      if is_self or (i & (i + 1)) == 0:  # care 0-index
        row.append(str(d.Rank))
        row.append(d.UserScreenName)
        for task in standing.TaskInfo:
          if task.TaskScreenName in d.TaskResults:
            # score = d.TaskResults[task.TaskScreenName].Score // 100
            penalty = d.TaskResults[task.TaskScreenName].Penalty
            elapsed_s = d.TaskResults[task.TaskScreenName].Elapsed // 1000 // 1000 // 1000
            row.append(f"+{penalty}\n{s2str(elapsed_s)}")
          else:
            row.append("")
        table.add_row(*row, style=Style(bgcolor="dark_green" if is_self else None))
        if is_self:
          break

    console.print(table)

  def get_language(self) -> LangKV:
    results = fetch_language(self.http_util)
    ret: LangKV = {}
    for item in results:
      ret[item.value] = item.text
    return ret

  def print_contest_list(self) -> bool:
    # self.login_website()
    result = fetch_list(self.http_util)
    from .AtCoder_printList import printData
    printData(result)
    return True

  def cid2url(self, cid: str) -> str:
    return f'{self._base_url}contests/{cid}'


def AtcoderGen(account: Account, provider: Provider2) -> BaseOj:
  http_util = provider.get(DI_HTTP)
  logger = provider.get(DI_LOGGER)
  oj: BaseOj = AtCoder(http_util=http_util, logger=logger, account=account, html_tag=HtmlTag(http_util))
  return oj



import click
import getpass
import logging
from rich.console import Console

from oi_cli2.cli.adaptor.ojman import OJManager
from oi_cli2.core.DI import DI_ACCMAN, DI_LOGGER, DI_PROVIDER
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.utils.account import AccountManager

console = Console(color_system='256', style=None)


@click.group()
@click.pass_context
def account(ctx):
  """Manage accounts"""
  from oi_cli2.utils.Provider2 import Provider2
  ctx.obj[DI_PROVIDER] = Provider2()


@account.command(name='list')
@click.pass_context
def list_command(ctx) -> None:
  """List all account"""
  provider = ctx.obj[DI_PROVIDER]
  am: AccountManager = provider.get(DI_ACCMAN)
  acc_list = am.get_list()
  for i in range(len(acc_list)):
    if i == 0 or acc_list[i].platform != acc_list[i - 1].platform:
      print(acc_list[i].platform)
    mark = ' '
    if acc_list[i].default:
      mark = '*'
    print(f'  {mark} {acc_list[i].account}')
  if len(acc_list) == 0:
    print("Account List is empty.")


@account.command()
@click.argument("platform")
@click.argument("account")
@click.option("-d", "--default", "default_", is_flag=True, help='Set account as default account in the oj platform.')
@click.pass_context
def new(ctx, platform, account, default_) -> None:
  """Create new account


  PLATFORM    Platform Name, (AtCoder,Codeforces)

  ACCOUNT     Account name
  """
  provider = ctx.obj[DI_PROVIDER]
  logger: logging.Logger = provider.get(DI_LOGGER)
  am: AccountManager = provider.get(DI_ACCMAN)
  password = getpass.getpass("Password:")
  if not am.new(platform=platform, account=account, password=password, default=default_):
    logger.error('New Account Failed.')
  else:
    logger.info('Success')

  # TODO support password in arg???


@account.command()
@click.argument("platform")
@click.argument("account")
@click.option("-p", "--password", "changepassword", is_flag=True, help='Change account password.')
@click.option("-d", "--default", "default_", is_flag=True, help='Set account as default account in the oj platform.')
@click.pass_context
def modify(ctx, platform, account, changepassword: bool, default_):
  """Modify a specific account default status or change password


  PLATFORM    Platform Name, (AtCoder,Codeforces)

  ACCOUNT     Account name
  """
  provider = ctx.obj[DI_PROVIDER]
  logger: logging.Logger = provider.get(DI_LOGGER)
  am: AccountManager = provider.get(DI_ACCMAN)
  if changepassword:
    password = getpass.getpass("Password:")
  else:
    password = None

  if not am.modify(platform=platform, account=account, password=password, default=default_):
    logger.error('Modify Account Failed.')
  else:
    logger.info('Success Modify')


@account.command()
@click.argument("platform")
@click.argument("account")
@click.pass_context
def delete(ctx, platform, account) -> bool:
  """Delete a specific account

  PLATFORM    Platform Name, (AtCoder,Codeforces)

  ACCOUNT     Account name
  """
  provider = ctx.obj[DI_PROVIDER]
  logger: logging.Logger = provider.get(DI_LOGGER)
  am: AccountManager = provider.get(DI_ACCMAN)
  if not am.delete(platform=platform, account=account):
    logger.error("Account not found")
    return False
  else:
    logger.info("Success Delete")
  return True


@account.command(name="test")
@click.argument("platform")
@click.argument("account")
@click.pass_context
def valid_account(ctx, platform: str, account: str) -> bool:
  """Test account login

  PLATFORM    Platform Name, (AtCoder,Codeforces)

  ACCOUNT     Account name
  """
  provider = ctx.obj[DI_PROVIDER]
  logger: logging.Logger = provider.get(DI_LOGGER)
  logger.debug(f'platform:{platform}')
  am: AccountManager = provider.get(DI_ACCMAN)
  acc = am.get_account(platform=platform, account=account)
  if acc is None:
    console.print(f'[red bold]Account [{account}] not found')
    return False

  try:
    oj: BaseOj = OJManager.createOj(platform=platform, account=acc, provider=provider)
  except Exception as e:
    logger.exception(e)
    raise e
  console.print(f"[green bold]{platform} Logging with {acc.account} ...")
  ok = oj.login_website(force=True)
  if ok:
    console.print(f"[green bold]Successful login.")
  else:
    console.print(f"[red bold]Login failed.")
  return ok



from dataclasses import dataclass, field
from enum import Enum
from typing import List

from oi_cli2.model.ParseProblemResult import ParsedProblemResult


class E_STATUS(str, Enum):
  NOT_SUBMITTED = ""
  AC = "AC"
  ERROR = "ERROR"


@dataclass
class ProblemMeta:
  id: str = ''
  url: str = ''
  name: str = ''
  passed: str = ''  # number of passed submission in contest
  score: int = 0
  status: E_STATUS = E_STATUS.NOT_SUBMITTED
  time_limit_msec: int = 0  # ms
  memory_limit_kb: int = 0  # mb
  contest_id: str = ''


@dataclass
class ContestMeta:
  id: str = ''
  url: str = ''
  problems: List[ProblemMeta] = field(default_factory=lambda: [])



# COLOR
RED = "\x1b[31m"
DEFAULT = "\x1b[39m"
GREEN = "\x1b[32m"
YELLOW = "\x1b[33m"
# FOLDER
# ROOT_PATH = os.path.dirname(os.path.realpath(__file__)) + '/'
DIST = 'dist'
TEST_FOLDER = 'TEST/'
TEMPLATEFOLDER = 'template/'
OT_FOLDER = '.oiTerminal'
# FILE
# CONFIG_FILE = ROOT_PATH + 'config.json'
STATE_FILE = 'state.json'
TEST_PY = 'test.py'
SUBMIT_PY = 'submit.py'
# LANG_CONFIG_FILE = ROOT_PATH + 'lang.json'
OT_LOG = 'log/oiTerminal.log'
USER_CONFIG_FILE = '_userConfig.json'
COOKIES_FILE = '_cookies.json'
ACCT_CONFIG_FILE = '_acctConfig.json'  # TODO split config json
TEMP_CONFIG_FILE = '_tempConfig.json'
# LOG
APP_NAME = 'oiTerminal'

IN_PREFIX = 'in.'
OUT_PREFIX = 'out.'

# CONFIG
FETCH_RESULT_INTERVAL = 2
CIPHER_KEY = 'oiTerminal'



from oi_cli2.core.DI import DI_DB_COOKIES, DI_HTTP
from oi_cli2.utils.HttpUtil import HttpUtil
from oi_cli2.utils.Provider2 import Provider2
from oi_cli2.utils.consts.ids import DB_COOKIES_ID
from oi_cli2.utils.db import JsonFileDB
from requests.utils import dict_from_cookiejar, cookiejar_from_dict


class HttpUtilCookiesHelper:

  def __init__(self) -> None:
    pass

  @staticmethod
  def save_cookie(provider: Provider2, platform: str, account: str) -> None:
    http_util: HttpUtil = provider.get(DI_HTTP)
    db: JsonFileDB = provider.get(DI_DB_COOKIES)
    data = db.load(DB_COOKIES_ID.cookies) or {}
    if platform not in data:
      data[platform] = {}
    data[platform][account] = dict_from_cookiejar(http_util.cookies)

    db.save(DB_COOKIES_ID.cookies, data)

  @staticmethod
  def load_cookie(provider: Provider2, platform: str, account: str) -> bool:
    http_util: HttpUtil = provider.get(DI_HTTP)
    db: JsonFileDB = provider.get(DI_DB_COOKIES)
    data = db.load(DB_COOKIES_ID.cookies) or {}
    if platform in data:
      if account in data[platform]:
        http_util.cookies.update(cookiejar_from_dict(data[platform][account]))
        return True
    return False



from typing import Any, Dict, cast
import click
from .reg_list import reg_list

# TODO inject account

platformNames = list(reg_list.keys())


def platformKey() -> Dict[str, Any]:
  res: Dict[str, Any] = {}
  for k in platformNames:
    res[k] = {}
  return res


platformKeys = platformKey()

# TODO auto generate by click
cliAccept = {
    "init": {},
    "config": {
        "account": {
            "list": {},
            "new": platformKey(),
            "modify": platformKey(),
            "delete": platformKey(),
            "test": platformKey(),  # TODO test login
        },
        "template": {
            "list": {},
            "new": platformKey(),
            "modify": platformKey(),
            "delete": platformKey(),
        }
    },
    # "problem": {
    #     "fetch": platformKey()
    # },
    "contest": {
        "list": platformKey(),
        "detail": platformKey(),
        "standing": platformKey(),
        "fetch": platformKey(),
        "open": platformKey(),
        "reg": platformKey(),
    },
    "lang": platformKey(),
    "test": {}, # TODO 换成基于click的自动化补全工具
    "submit": {},
    "result": {},
}


@click.command(name="completion")
@click.argument('cmds', nargs=-1)  # unlimited args
def completion_command(cmds):  # (oi, xxx, xxx)
  # Debug
  # with open('/tmp/out','a+') as f:
  #   f.write(str(cmds))
  ptr = cliAccept
  for i in range(1, len(cmds)):
    key = cmds[i]
    if key in ptr:
      ptr = cast(Dict[str, Any], ptr[key])
    else:
      print()
      return
  for k in ptr:
    print(k, end=' ')
  print()



import logging
from typing import List

import click
from oi_cli2.core.DI import DI_DB, DI_LOGGER, DI_TEMPMAN

from oi_cli2.model.Template import Template
from oi_cli2.utils.template import TemplateManager


@click.group()
@click.pass_context
def template(ctx):
  """Manage templates"""
  db = ctx.obj[DI_DB]
  ctx.obj[DI_TEMPMAN] = TemplateManager(db)


@template.command(name='list')
@click.option('-d', '--detail', is_flag=True, help="display config detail")
@click.pass_context
def list_command(ctx, detail: bool):
  """List all templates"""
  tm: TemplateManager = ctx.obj[DI_TEMPMAN]
  temp_list: List[Template] = tm.get_list()
  for i in range(len(temp_list)):
    if i == 0 or temp_list[i].platform != temp_list[i - 1].platform:
      print(temp_list[i].platform)
    mark = ' '
    if temp_list[i].default:
      mark = '*'
    print(f'  {mark} {temp_list[i].alias}')
    if detail:
      print(f'  \tCompile command    : {temp_list[i].compilation}')
      print(f'  \tExecute command    : {temp_list[i].execute}')
      print(f'  \tTemplate file Path : {temp_list[i].path}')
      print(f'  \tUpload language id : {temp_list[i].uplang}')
  if len(temp_list) == 0:
    print("Template list is empty.")


@template.command()
@click.pass_context
@click.argument('platform')
@click.argument('name')
@click.argument('path')
@click.argument('compile')
@click.argument('execute')
@click.argument('langid')
def new(ctx, platform, name, path, compile, execute, langid) -> None:
  """Create new template

  PLATFORM    Platform Name, (AtCoder,Codeforces)

  NAME        Custom template name

  PATH        Your template file path

  COMPILE     Compile command

  EXECUTE     Execute command

  LANGID      Upload language id(`oi lang <platform>`)"""
  tm: TemplateManager = ctx.obj[DI_TEMPMAN]
  logger: logging.Logger = ctx.obj[DI_LOGGER]
  logger.debug(f"{platform}, {name}, {path}, {compile}, {execute}, {langid}")
  tm.add_template(platform=platform, alias=name, path=path, compilation=compile, execute=execute, uplang=langid)


@template.command()
@click.pass_context
@click.argument('platform')
@click.argument('name')
def delete(ctx, platform, name) -> None:
  """Delete a specific template"""
  tm: TemplateManager = ctx.obj[DI_TEMPMAN]
  tm.delete_template(platform, name)


@template.command()
@click.pass_context
@click.argument('platform')
@click.argument('name')
@click.option('-n', '--name', 'newname', help='Change template name')
@click.option('-p', '--path', help='Change template path')
@click.option('-c', '--compile', help='Change compile command')
@click.option('-e', '--execute', help='Change execute command')
@click.option('-l', '--langid', help='Change upload language id')
@click.option('-d', '--default', is_flag=True, help='Set as default template')
def modify(ctx, platform, name, newname, path, compile, execute, langid, default) -> None:
  """Update current template

  PLATFORM    Platform Name, (AtCoder,Codeforces)

  NAME        Custom template name
  """
  tm: TemplateManager = ctx.obj[DI_TEMPMAN]
  logger: logging.Logger = ctx.obj[DI_LOGGER]
  logger.debug(f"{platform}, {name}, {path}, {compile}, {execute}, {langid},{default}")
  tm.update_template(platform=platform,
                     alias=name,
                     newalias=newname,
                     path=path,
                     compilation=compile,
                     execute=execute,
                     uplang=langid,
                     default=default)



# TODO remove full file replace with new implement
import typing
from bs4 import element

from oi_cli2.abstract.HtmlTagAbstract import HtmlTagAbstract
from oi_cli2.utils.HttpUtil import HttpUtil


class HtmlTag(HtmlTagAbstract):

  def __init__(self, http_util: HttpUtil):
    super().__init__(http_util)

  @typing.no_type_check
  def update_tag(self, tag, oj_prefix: str, update_style=None):
    """
        :param tag: 一个顶级tag，从这个tag递归遍历所有子tag，寻找需要修改url的节点
        :param oj_prefix: 原oj的静态文件前缀
        :param update_style: 不为空的话，递归修改内联style
        :return: 成功返回原tag，失败返回None
        """
    if type(tag) == element.Tag:
      for child in tag.descendants:
        if type(child) != element.Tag:
          continue
        if update_style:
          child['style'] = update_style
        if child.name == 'a' and child.get('href'):
          if not child.get('class'):
            child['class'] = ()
          child['class'] += (HtmlTag.TagDesc.ANCHOR.value, )
          child['target'] = ('_blank', '_parent')
          child['href'] = self.http_util.abs_url(child.get('href'), oj_prefix=oj_prefix)[-1]
        if child.name == 'img' and child.get('src'):
          if not child.get('class'):
            child['class'] = ()
          child['class'] += (HtmlTag.TagDesc.IMAGE.value, )
          child['src'] = self.http_util.abs_url(child.get('src'), oj_prefix=oj_prefix)[-1]
    return tag



import json
import logging
import os
from typing import Any, Optional


# write through , read through
class JsonFileDB:
  file_path: str

  def __init__(self, file_path: str, logger: logging.Logger):
    # check exist
    # if not exist create
    self.file_path: str = file_path
    self.logger: logging.Logger = logger

  # 不支持class 实例, 需要 调用者处理实例和dict之间转换 出入
  def save(self, col_name: str, obj: Any) -> bool:
    if not os.path.exists(self.file_path):
      os.mknod(self.file_path)

    with open(self.file_path, "r") as json_file:
      try:
        data = json.load(json_file)
      except Exception:
        data = {}

    with open(self.file_path, "w") as json_file:
      data[col_name] = obj
      json.dump(data, json_file, indent=2)
    return True

  def load(self, col_name: str) -> Optional[Any]:
    if not os.path.exists(self.file_path):
      os.mknod(self.file_path)

    with open(self.file_path, "r") as json_file:
      try:
        data = json.load(json_file)
      except json.JSONDecodeError:  # empty file or wrong file, type(e).__name__
        data = {}
      except Exception as e:
        raise e

      return data.get(col_name)



from typing import Callable, Dict, List
from oi_cli2.model.Account import Account
from oi_cli2.model.BaseOj import BaseOj
from oi_cli2.utils.Provider2 import Provider2

T_OJFn = Callable[[Account, Provider2], BaseOj]


class OJManager:
  _ojFn: Dict[str, T_OJFn] = {}
  # _ojObj: Dict[str, BaseOj] = {}

  @staticmethod
  def regOj(platform: str, fn: T_OJFn) -> bool:
    if platform in OJManager._ojFn:
      return False
    OJManager._ojFn[platform] = fn
    return True

  @staticmethod
  def createOj(platform: str, account: Account, provider: Provider2) -> BaseOj:
    if platform in OJManager._ojFn:
      return OJManager._ojFn[platform](account, provider)
    else:
      raise Exception(f'Unknown Platform [{platform}]')



from typing import List

from oi_cli2.model.Analyze import Analyze
from oi_cli2.utils.analyze import AnalyzeManager

# TODO 动态注册不同平台


def analyze_list(tm: AnalyzeManager):
  analyze_list: List[Analyze] = tm.get_list()
  for i in range(len(analyze_list)):
    if i == 0 or analyze_list[i].platform != analyze_list[i - 1].platform:
      print(analyze_list[i].platform)
    mark = ' '
    if analyze_list[i].default:
      mark = '*'
    print(f'\t {mark} {analyze_list[i].template_alias} {analyze_list[i].submit_lang}')

  if len(analyze_list) == 0:
    print("Analyze list is empty.")


def analyze_new(tm: AnalyzeManager):
  platforms = ['Codeforces', 'AtCoder']
  for i in range(len(platforms)):
    print(f"{i + 1}) {platforms[i]}")
  try:
    index = int(input("> "))
  except Exception:
    print("input error")
    return

  if 0 < index <= len(platforms):
    platform = platforms[index - 1]
  else:
    print("input error")
    return

  submit_lang = input('submit_lang:')
  template_alias = input('template_alias:')
  class_path = input('class_path:')

  tm.add_analyze(platform, submit_lang, template_alias, class_path)


def analyze_modify(tm: AnalyzeManager):
  analyze_list = tm.get_list()

  for i in range(len(analyze_list)):
    if i == 0 or analyze_list[i].platform != analyze_list[i - 1].platform:
      print(analyze_list[i].platform)
    mark = ' '
    item = analyze_list[i]
    if item.default:
      mark = '*'
    print(f'\t {mark} {i}) {item.template_alias}')
    print(f'\t\t submit_lang: {item.submit_lang}')
    print(f'\t\t class_path:  {item.class_path}')

  try:
    acc_index = int(input("> "))
  except Exception:
    print("input error")

  if acc_index < 0 or acc_index >= len(analyze_list):
    print("input error")
    return

  print("1) Change Analyze template")
  print("2) Change Analyze submit language")
  print("3) Change Analyze class path")
  print("4) Set as Default")
  print("5) Delete")

  try:
    index = int(input("> "))
  except (Exception):
    print("input error")

  if index == 1:
    tm.modify_template_alias(acc_index, input("Enter template_alias:"))
  elif index == 2:
    tm.modify_submit_lang(acc_index, input("Enter submit lang:"))
  elif index == 3:
    tm.modify_class_path(acc_index, input("Enter class path:"))
  elif index == 4:
    tm.set_default(acc_index)
  elif index == 5:
    tm.delete_analyze(acc_index)


def analyze(db):
  print("1) Analyze List")
  print("2) New Analyze")
  print("3) Modify Analyze")
  try:
    index = int(input("> "))
  except (Exception):
    print("input error")

  tm = AnalyzeManager(db)

  if index == 1:
    analyze_list(tm)
  elif index == 2:
    analyze_new(tm)
  elif index == 3:
    analyze_modify(tm)
  else:
    print("input error")
